!+
! Module: histogram_utils
!
! Authors:  Ian Stewart - SSC/LUX
!          
! Description: Contains some assorted routines.
! 
! Comments: TODO:
! 
! $Id: module.f90,v 1.6 1998/05/27 13:45:39 gvacanti Exp $
!- 
module histogram_utils

  use types
  use errorHandling

!  use array_utils	! for getBinNumber, smallestTrueIndex
!use temp_ssclib
  use ssc_misc		! for fracVal

  implicit none 

  private 

  type, public :: weightedHistoSingleType
    real(single) ::&
      xLo,&
      xHi,&
      yAve,&
      yStdDev,&
      yUncert
    integer(int32) ::&
      n
  end type weightedHistoSingleType
  type, public :: weightedHistoDoubleType
    real(double) ::&
      xLo,&
      xHi,&
      yAve,&
      yStdDev,&
      yUncert
    integer(int32) ::&
      n
  end type weightedHistoDoubleType

  public initializeWeightedHistoSingle, initializeWeightedHistoDouble
  public makeHistogram
  public makeWeightedHistogram
  public calcBinEdges
!  public accumulateWeightedHisto

  interface makeHistogram
!    module procedure makeHistogramInt8, makeHistogramInt16, makeHistogramInt32
    module procedure makeHistogramEdgesSingle, makeHistogramBoundsSingle
    module procedure makeHistoEdgesSingleWeights
    module procedure makeHistogramEdgesDouble, makeHistogramBoundsDouble
  end interface

  interface makeWeightedHistogram
    module procedure makeWeightedHistogramSingle, makeWeightedHistogramDouble
  end interface

contains 

  !---------------------------------------------------------------------
  subroutine initializeWeightedHistoDouble(histo)
  !---------------------------------------------------------------------

    implicit none

    type(weightedHistoDoubleType), intent(out) :: histo(:)

    !-------------------------------------------------------------------
    ! Start

    histo%xLo = 0.0d0
    histo%xHi = 0.0d0
    histo%yAve = 0.0d0
    histo%yStdDev = 0.0d0
    histo%yUncert = 0.0d0
    histo%n = 0

  end subroutine initializeWeightedHistoDouble

  !---------------------------------------------------------------------
  subroutine initializeWeightedHistoSingle(histo)
  !---------------------------------------------------------------------

    implicit none

    type(weightedHistoSingleType), intent(out) :: histo(:)

    !-------------------------------------------------------------------
    ! Start

    histo%xLo = 0.0
    histo%xHi = 0.0
    histo%yAve = 0.0
    histo%yStdDev = 0.0
    histo%yUncert = 0.0
    histo%n = 0

  end subroutine initializeWeightedHistoSingle

  !---------------------------------------------------------------------
  function copyHistoDbleToSingle(histoDouble) result(histoSingle)
  !---------------------------------------------------------------------

    implicit none

    !-------------------------------------------------------------------
    ! Start

    type(weightedHistoDoubleType), intent(in) :: histoDouble(:)
    type(weightedHistoSingleType) :: histoSingle(size(histoDouble))

    !-------------------------------------------------------------------
    ! Start

    histoSingle%xLo     = real(histoDouble%xLo, single)
    histoSingle%xHi     = real(histoDouble%xHi, single)
    histoSingle%yAve    = real(histoDouble%yAve, single)
    histoSingle%yStdDev = real(histoDouble%yStdDev, single)
    histoSingle%yUncert = real(histoDouble%yUncert, single)
    histoSingle%n       = histoDouble%n

  end function copyHistoDbleToSingle

!  !---------------------------------------------------------------------
!  subroutine accumulateWeightedHisto(sumHisto, newHisto)
!  !---------------------------------------------------------------------
!
!    implicit none
!
!    type(weightedHistoSingleType), intent(inout) :: sumHisto(:)
!    type(weightedHistoSingleType), intent(in)    :: newHisto(size(sumHisto))
!
!    !-------------------------------------------------------------------
!    ! Start
!
!  end subroutine accumulateWeightedHisto
!
!  !---------------------------------------------------------------------
!  subroutine makeHistogramInt32(inVector, binContents&
!  , underflows, overflows, status)
!  !---------------------------------------------------------------------
!
!    implicit none
!
!    integer(int32), intent(in)            :: inVector(:)
!    integer,        pointer               :: binContents(:)
!    integer,        intent(out), optional :: underflows, overflows, status
!
!!    integer(int32) :: lEdge, rEdge
!!    integer :: hsize, i, localUnderflows, localOverflows, localStatus
!!    real(single) :: frac
!!    real(single), allocatable :: binEdges(:)
!
!    !-------------------------------------------------------------------
!    ! Start
!
!    localStatus = 0 ! default.
!
!    if (associated(binContents))
!
!!    hsize = size(binContents)
!!    allocate(binEdges(hsize + 1))
!!
!!    do i = 1, hsize + 1
!!      frac = (i - 1) / real(hsize)
!!      binEdges(i) = lEdge * (1.0 - frac) + rEdge * frac
!!    end do
!!
!!    call makeHistogramKernel(dble(inVector), binContents, dble(binEdges)&
!!    , localUnderflows, localOverflows, localStatus)
!!
!!    if (present(underflows)) underflows = localUnderflows
!!    if (present(overflows)) overflows = localOverflows
!!    if (present(status)) status = localStatus
!
!  end subroutine makeHistogramInt32

  !---------------------------------------------------------------------
  subroutine makeHistogramBoundsSingle(inVector, binContents, lEdge, rEdge&
  , underflows, overflows)
  !---------------------------------------------------------------------

    implicit none

    real(single),   intent(in)            :: inVector(:)
    integer(int32), intent(out)           :: binContents(:)
    real(single),   intent(in)            :: lEdge, rEdge
    integer(int32), intent(out), optional :: underflows, overflows

    integer :: hsize, i, localUnderflows, localOverflows
    real(single) :: frac
    real(single), allocatable :: binEdges(:)

    !-------------------------------------------------------------------
    ! Start

    hsize = size(binContents)
    allocate(binEdges(hsize + 1))

    do i = 1, hsize + 1
      frac = (i - 1) / real(hsize)
      binEdges(i) = lEdge * (1.0 - frac) + rEdge * frac
    end do

    call makeHistogramKernel(dble(inVector), binContents, dble(binEdges)&
      , localUnderflows, localOverflows)

    if (present(underflows)) underflows = localUnderflows
    if (present(overflows)) overflows = localOverflows

  end subroutine makeHistogramBoundsSingle

  !---------------------------------------------------------------------
  subroutine makeHistogramEdgesSingle(inVector, binContents, binEdges&
  , underflows, overflows)
  !---------------------------------------------------------------------

    implicit none

    real(single),   intent(in)            :: inVector(:)
    integer(int32), intent(out)           :: binContents(:)
    real(single),   intent(in),  optional :: binEdges(size(binContents)+1)
    integer(int32), intent(out), optional :: underflows, overflows

    integer :: hsize, binEdge, localUnderflows, localOverflows
    real(single) :: frac, minx, maxx
    real(single), allocatable :: myBinEdges(:)

    !-------------------------------------------------------------------
    ! Start

    hsize = size(binContents)
    minx = minval(inVector)
    maxx = maxval(inVector)

    allocate(myBinEdges(hsize + 1))
    if (present(binEdges)) then
      myBinEdges = binEdges(1:hsize + 1)

    else
      do binEdge = 1, hsize + 1
        frac = (binEdge - 1) / real(hsize)
        myBinEdges(binEdge) = minx * (1.0 - frac) + maxx * frac
      end do
    end if

    call makeHistogramKernel(dble(inVector), binContents, dble(myBinEdges)&
    , localUnderflows, localOverflows)

    if (present(underflows)) underflows = localUnderflows
    if (present(overflows)) overflows = localOverflows

  end subroutine makeHistogramEdgesSingle

  !---------------------------------------------------------------------
  subroutine makeHistoEdgesSingleWeights(inVector, binContents, binEdges&
  , weights, underflows, overflows, binErrs)
  !---------------------------------------------------------------------

    implicit none

    real(single), intent(in)            :: inVector(:)
!    integer,      intent(out)           :: binContents(:)
    real(single), intent(out)           :: binContents(:)
    real(single), intent(in),  optional :: binEdges(size(binContents)+1),&
                                           weights(size(inVector))
    real(single), intent(out), optional :: underflows, overflows,&
                                           binErrs(size(binContents))

    integer :: hsize, binEdge
    real(single) :: frac, minx, maxx
    real(single), allocatable :: myBinEdges(:), myWeights(:)
    real(double), allocatable :: binContentsDble(:), localBinErrsDble(:)
    real(double) :: localUnderflows, localOverflows

    !-------------------------------------------------------------------
    ! Start

    hsize = size(binContents)
    minx = minval(inVector)
    maxx = maxval(inVector)

    allocate(myBinEdges(hsize + 1))
    if (present(binEdges)) then
      myBinEdges = binEdges(1:hsize + 1)

    else
      do binEdge = 1, hsize + 1
        frac = (binEdge - 1) / real(hsize)
        myBinEdges(binEdge) = minx * (1.0 - frac) + maxx * frac
      end do
    end if

    allocate(myWeights(size(inVector)))
    if (present(weights)) then
      myWeights = weights
    else
      myWeights = 1.0
    end if

    allocate(binContentsDble(hsize), localBinErrsDble(hsize))
    call makeHistogramKernelWeights(dble(inVector), dble(myWeights), dble(myBinEdges)&
    , binContentsDble, localUnderflows, localOverflows, localBinErrsDble)

    binContents = binContentsDble

    if (present(underflows)) underflows = localUnderflows
    if (present(overflows)) overflows = localOverflows
    if (present(binErrs)) binErrs = localBinErrsDble

  end subroutine makeHistoEdgesSingleWeights

  !---------------------------------------------------------------------
  subroutine makeHistogramBoundsDouble(inVector, binContents, lEdge, rEdge&
  , underflows, overflows)
  !---------------------------------------------------------------------

    implicit none

    real(double),   intent(in)            :: inVector(:)
    integer(int32), intent(out)           :: binContents(:)
    real(double),   intent(in)            :: lEdge, rEdge
    integer(int32), intent(out), optional :: underflows, overflows

    integer :: hsize, i, localUnderflows, localOverflows
    real(double) :: frac
    real(double), allocatable :: binEdges(:)

    !-------------------------------------------------------------------
    ! Start

    hsize = size(binContents)
    allocate(binEdges(hsize + 1))

    do i = 1, hsize + 1
      frac = (i - 1) / real(hsize)
      binEdges(i) = lEdge * (1.0 - frac) + rEdge * frac
    end do

    call makeHistogramKernel(inVector, binContents, binEdges&
    , localUnderflows, localOverflows)

    if (present(underflows)) underflows = localUnderflows
    if (present(overflows)) overflows = localOverflows

  end subroutine makeHistogramBoundsDouble

  !---------------------------------------------------------------------
  subroutine makeHistogramEdgesDouble(inVector, binContents, binEdges&
  , underflows, overflows)
  !---------------------------------------------------------------------

    implicit none

    real(double),   intent(in)            :: inVector(:)
    integer(int32), intent(out)           :: binContents(:)
    real(double),   intent(in),  optional :: binEdges(size(binContents)+1)
    integer(int32), intent(out), optional :: underflows, overflows

    integer :: hsize, binEdge, localUnderflows, localOverflows
    real(double) :: frac, minx, maxx
    real(double), allocatable :: myBinEdges(:)

    !-------------------------------------------------------------------
    ! Start

    hsize = size(binContents)
    minx = minval(inVector)
    maxx = maxval(inVector)

    allocate(myBinEdges(hsize + 1))
    if (present(binEdges)) then
      myBinEdges = binEdges(1:hsize + 1)

    else
      do binEdge = 1, hsize + 1
        frac = (binEdge - 1) / real(hsize)
        myBinEdges(binEdge) = minx * (1.0 - frac) + maxx * frac
      end do
    end if

    call makeHistogramKernel(inVector, binContents, myBinEdges&
    , localUnderflows, localOverflows)

    if (present(underflows)) underflows = localUnderflows
    if (present(overflows)) overflows = localOverflows

  end subroutine makeHistogramEdgesDouble

  !---------------------------------------------------------------------
  subroutine makeWeightedHistogramSingle(xVals, yVals, xBinEdges, histo&
  , underflows, overflows)
  !---------------------------------------------------------------------

    implicit none

    real(single),                  intent(in)  :: xVals(:),&
                                                  yVals(size(xVals))
    type(weightedHistoSingleType), intent(out) :: histo(:)
    real(single),                  intent(in)  :: xBinEdges(size(histo)+1)
    integer,                       intent(out) :: underflows, overflows

    type(weightedHistoDoubleType), allocatable :: histoDble(:)

    !-------------------------------------------------------------------
    ! Start

    allocate(histoDble(size(histo)))
    call makeWeightedHistogramDouble(real(xVals, double)&
      , real(yVals, double), real(xBinEdges, double), histoDble&
      , underflows, overflows)

    histo = copyHistoDbleToSingle(histoDble)

  end subroutine makeWeightedHistogramSingle

  !---------------------------------------------------------------------
  subroutine makeWeightedHistogramDouble(xVals, yVals, xBinEdges, histo&
  , underflows, overflows)
  !---------------------------------------------------------------------
  ! The idea of this routine is not so much to make histograms of the
  ! frequency of occurrence of a certain quantity X, which is sometimes
  ! what is desired; but rather to take in pairs of values (X,Y) and
  ! calculate a series of bin averages of Y at values of X; the output
  ! value at bin i being calculated by averaging all Y values for which
  ! the paired X falls within the bin. Also of interest is the standard
  ! deviation of the values in the bin. Note that this routine still
  ! calculates a frequency histogram: this is returned in the 'n' field
  ! of the histogram data structure. 
  !
  ! I didn't include notes to explain what I was doing at the time when
  ! I wrote this routine, and I am not 100% sure now what I intended with
  ! the 'yUncert' field. I think my reasoning was as follows: the best
  ! estimate of the uncertainty in an individual measurement of Y is the
  ! standard deviation, the square root of the variance, which is
  !
  !	         1   _N
  !	var_Y = --- >_   (Y_i - <Y>)^2,
  !	         N   i=1
  !
  ! where the sum is just over those Ys for which X falls within the bin.
  ! If we substitute this expression into the usual error propagation
  ! relation, the uncertainty in <Y> becomes
  !
  !	                1    _N
  !	sigma^2_<Y> = ----- >_   var_Y
  !	               N^2   i=1
  !
  !	                1    _N
  !	            = ----- >_   (Y_i - <Y>)^2,
  !	               N^2   i=1
  !
  ! which is the formula used for the 'yUncert' field.
  !
  ! IMS 2006-11-09.
  !---------------------------------------------------------------------

!*** something close to this should become the new histogramKernel.

!*** check (and write some explanations) for the various values of histo.

    implicit none

    real(double),          intent(in)  :: xVals(:), yVals(size(xVals))
    type(weightedHistoDoubleType), intent(out) :: histo(:)
    real(double),          intent(in)  :: xBinEdges(size(histo)+1)
    integer,               intent(out) :: underflows, overflows

    integer :: vsize, numBins, bin
!, i
    real(double) :: minx, maxx
!, x
    logical, allocatable :: mask(:)

    !-------------------------------------------------------------------
    ! Start

!    binContents = 0
call initializeWeightedHistoDouble(histo)
    underflows = 0
    overflows = 0

    vsize = size(xVals)
    numBins = size(histo)

    if (vsize <= 1)&
call error('',"")
    if (numBins <= 1)&
call error('',"")
!    if (size(xBinEdges) .ne. numBins + 1)&
!call error('',"")
    if (any(xBinEdges(1:numBins) >= xBinEdges(2:numBins+1)))&
call error('',"")

    allocate(mask(vsize))

    minx = xBinEdges(1)
    maxx = xBinEdges(numBins + 1)

    histo%xLo = xBinEdges(:numBins)
    histo%xHi = xBinEdges(2:)

    do bin = 1, numBins
      if (bin == numBins) then
        mask = (xVals >= histo(bin)%xLo .and. xVals <= histo(bin)%xHi)
      else
        mask = (xVals >= histo(bin)%xLo .and. xVals < histo(bin)%xHi)
      end if

      histo(bin)%n = count(mask)
      if (histo(bin)%n > 0) then
        histo(bin)%yAve = sum(yVals, mask) / real(histo(bin)%n)
      else
        histo(bin)%yAve = 0.0
      end if

      histo(bin)%yStdDev = sum((yVals - histo(bin)%yAve)&
        * (yVals - histo(bin)%yAve), mask)

      if (histo(bin)%n > 0) then
        histo(bin)%yStdDev = sqrt(histo(bin)%yStdDev / real(histo(bin)%n))
        histo(bin)%yUncert = histo(bin)%yStdDev / sqrt(real(histo(bin)%n))
      else
        histo(bin)%yStdDev = 0.0
        histo(bin)%yUncert = 0.0
      end if
    end do

    underflows = count(xVals < minx)
    overflows  = count(xVals > maxx)
!**** not going to bother to trap underflows or overflows (or histo%n for that matter) being larger than HUGE.

  end subroutine makeWeightedHistogramDouble

  !---------------------------------------------------------------------
  subroutine makeHistogramKernel(inVector, binContents, binEdges&
  , underflows, overflows)
  !---------------------------------------------------------------------

    implicit none

    real(double),   intent(in)  :: inVector(:)
    integer(int32), intent(out) :: binContents(:)
    real(double),   intent(in)  :: binEdges(size(binContents)+1)
    integer(int32), intent(out) :: underflows, overflows

    integer :: vsize, hsize, i, bin
    real(double) :: minx, maxx, x

    !-------------------------------------------------------------------
    ! Start

    binContents = 0
    underflows = 0
    overflows = 0

    vsize = size(inVector)
    hsize = size(binContents)

    if (vsize <= 0)&
call error('zeroSizeInVector',"makeHistogram:")
    if (hsize <= 0)&
call error('zeroSizeHistogram',"makeHistogram:")
!do i = 1, hsize
!print *, binEdges(i), (binEdges(i) >= binEdges(i+1))
!end do
!print *, binEdges(hsize+1)
    if (any(binEdges(1:hsize) >= binEdges(2:hsize+1)))&
call error('badOrderBinEdges',"makeHistogram:")

    minx = binEdges(1)
    maxx = binEdges(hsize + 1)

!    binContents = 0
!    underflows = 0
!    overflows = 0
    do i = 1, vsize
      x = inVector(i)
      if (x < minx) then
!        if (underflows .ge. huge(underflows)) then
!          status = -5
!  return
!        end if
        underflows = underflows + 1

      else if (x > maxx) then
!        if (overflows .ge. huge(overflows)) then
!          status = -6
!  return
!        end if
        overflows = overflows + 1

      else
!!!bin = getBinNumber(binEdges, x) 
        do bin = hsize, 1, -1
          if (x >= binEdges(bin)) then
!            if (binContents(bin) .ge. huge(binContents)) then
!              status = -7
!  return
!            end if
            binContents(bin) = binContents(bin) + 1
        exit
          end if
        end do
      end if
    end do

  end subroutine makeHistogramKernel

  !---------------------------------------------------------------------
  subroutine makeHistogramKernelWeights(inVector, weights, binEdges&
  , binContents, underflows, overflows, binErrs)
  !---------------------------------------------------------------------

    implicit none

    real(double), intent(in)  :: inVector(:),&
                                 weights(size(inVector))
    real(double), intent(out) :: binContents(:),&
                                 underflows,&
                                 overflows,&
                                 binErrs(size(binContents))
    real(double), intent(in)  :: binEdges(size(binContents)+1)

    integer :: vsize, hsize, i, bin
    real(double) :: minx, maxx, x, avWeight
    integer, allocatable :: numElements(:)

    !-------------------------------------------------------------------
    ! Start

    binContents = 0.0d0
    underflows = 0.0d0
    overflows = 0.0d0
    binErrs = 0.0d0

    vsize = size(inVector)
    hsize = size(binContents)

    if (vsize <= 1)&
call error('',"")
    if (hsize <= 1)&
call error('',"")
    if (size(binEdges) .ne. hsize + 1)&
call error('',"")
    if (any(binEdges(1:hsize) >= binEdges(2:hsize+1)))&
call error('',"")

    minx = binEdges(1)
    maxx = binEdges(hsize + 1)

!    binContents = 0.0d0
!    underflows = 0.0d0
!    overflows = 0.0d0
    allocate(numElements(hsize))
    numElements = 0
    do i = 1, vsize
      x = inVector(i)
      if (x < minx) then
!        if (underflows .ge. huge(underflows)) then
!          status = -5
!  return
!        end if
        underflows = underflows + weights(i)

      else if (x > maxx) then
!        if (overflows .ge. huge(overflows)) then
!          status = -6
!  return
!        end if
        overflows = overflows + weights(i)

      else
!!!bin = l_getBinNumberDouble(binEdges, x) 
        do bin = hsize, 1, -1
          if (x .ge. binEdges(bin)) then
!            if ( binContents(bin) >= huge(binContents)&
!            .or. numElements(bin) >= huge(numElements)) then
!              status = -7
!  return
!            end if
            numElements(bin) = numElements(bin) + 1
            binContents(bin) = binContents(bin) + weights(i)
        exit
          end if
        end do
      end if
    end do

    do bin = 1, hsize
      avWeight = binContents(bin) / real(numElements(bin), double)
      binErrs(bin) = sqrt(real(numElements(bin), double)) * avWeight
    end do

  end subroutine makeHistogramKernelWeights

  !---------------------------------------------------------------------
  subroutine calcBinEdges(binSize, inVector, binEdges)
  !---------------------------------------------------------------------

    implicit none

    real(single), intent(in) :: binSize, inVector(:)
    real(single), pointer    :: binEdges(:)


    real(single) :: minX, maxX, overShoot
    integer :: numBins, bin

    !-------------------------------------------------------------------
    ! Start

    if (binSize <= 0.0)&
call error('badHistoBinSize',"calcBinEdges:")

    minX = minval(inVector)
    maxX = maxval(inVector)

    numBins = max(1, ceiling((maxX - minX) / binSize))
    ! so there is at least 1 bin, even if all inVector values are the same.

    allocate(binEdges(numBins+1))
    overShoot = ((numBins * binSize) - (maxX - minX)) / 2.0
!print *, binSize, minX, maxX, numBins, overShoot

    do bin = 1, numBins
      binEdges(bin) = fracval(minX - overShoot, maxX + overShoot&
        , (bin - 1) / real(numBins)) ! in ssclib/ssc_misc.
    end do
    binEdges(numBins+1) = maxX + overShoot

  end subroutine calcBinEdges

  !---------------------------------------------------------------------
  subroutine template
  !---------------------------------------------------------------------

    implicit none

    !-------------------------------------------------------------------
    ! Start

  end subroutine template

end module histogram_utils
