  !---------------------------------------------------------------------
  function brentsMethod_origf77(ax, bx, cx, f, tol, xmin)
  !---------------------------------------------------------------------

    integer ITMAX
    real brentsMethod_origf77, brent, ax, bx, cx, tol, xmin, f, CGOLD, ZEPS
    external f
    parameter (ITMAX=100, CGOLD=0.3819660, ZEPS=1.0e-10)

    !-------------------------------------------------------------------
    ! Start

    a = min(ax,cx) ! a and b must be in ascending order, although the
    b = max(ax,cx) ! abcissas need not be.
    v = bx
    w = v
    x = v
    e = 0.0 ! This will be the distance moved on the step before last.
    fx = f(x)
    fv = fx
    fw = fx
    do iter = 1, ITMAX
      xm = 0.5 * (a + b)
      tol1 = tol * abs(x) + ZEPS
      tol2 = 2.0 * tol1
      if (abs(x - xm) <= (tol2 - 0.5 * (b - a)) goto 3 ! test for done
      if (abs(e) > tol1) then ! construct a trial parabolic fit
        r = (x - w) * (fx - fv)
        q = (x - v) * (fx - fw)
        p = (x - v) * q - (x - w) * r
        q = 2.0 * (q - r)
        if (q > 0.0) p = -p
        q = abs(q)
        etemp = e
        e = d
        if (abs(p) >= abs(0.5 * q * etemp) .or. p <= q * (a - x)
*         .or. p >= q * (b - x)) goto 1
! The above conditions determine the acceptability of the parabolic fit.
! Here it is ok:
        d = p / q ! ie, take the parabolic step.
        u = x + d
        if (u - a < tol2 .or. b - u < tol2) d = sign(tol1, xm - x)
        goto 2 ! ie, skip over the golden section step.
      end if

1     if (x >= xm) then ! we arrive here at the golden section step, which
      ! we take into the larger of the two segments.
        e = a - x
      else
        e = b - x
      end if

      d = CGOLD * e ! take the golden section step

2     if (abs(d) >= tol1) then ! arrive here with d computed either from the
      ! parabolic fit, or else from the golden section.
        u = x + d
      else
        u = x + sign(tol1, d)
      end if

      fu = f(u) ! this is the one function evaluation per iteration. We now
      ! have to decide what to do with our function evaluation. Housekeeping
      ! follows:

      if (fu <= fx) then
        if (u >= x) then
          a = x
        else
          b = x
        end if
        v = w
        fv = fw
        w = x
        fw = fx
        x = u
        fx = fu
      else
        if (u < x) then
          a = u
        else
          b = u
        end if

        if (fu <= fw .or. w == x) then
          v = w
          fv = fw
          w = u
          fw = fu
        else if (fu <= fv .or. v == x .or. v == w) then
          v = u
          fv = fu
        end if
      end if ! housekeeping finished. Back for another iteration.
    end do

    pause 'brent exceed max iterations'

3   xmin = x ! arrive here ready to exit with best values.

    brentsMethod_origf77 = fx
    return

  end function brentsMethod_origf77

