program testarrayutils
  use errorhandling
  use types
  use dal		! For INTEGER_NULL
  use array_utils
  use test_utils	! for isNearlyEqual()

  implicit none

  ! interface chkAryStatInfo
  !   module procedure chkAryStatInfoDouble2d	! This causes an error.
  ! end interface

  call test_getAxesFromIndices
  call test_getIndicesFromAxes
  call test_calcIndicesFromIndices
  call test_invertMask
  call test_sumInt32
  call test_getAryEdgesInfo
  call test_getAryStatInfo
  call test_getAnnularMaskAry

contains

  !---------------------------------------------------------------------
  subroutine test_getAxesFromIndices
  !---------------------------------------------------------------------
  ! Tests of getAxesFromIndices() in array_utils.

    implicit none

    character(22)  :: errHead

    integer :: iLbound1d(1), iLbound2d(2), iLbound3d(3) &
             , uLbound1d(1), uLbound2d(2), uLbound3d(3)
    real(double) :: axes1dDbl(1),    axes2dDbl(2),    axes3dDbl(3) &
                  , indices1dDbl(1), indices2dDbl(2), indices3dDbl(3) &
                  , lEdge1dDbl(1),   lEdge2dDbl(2),   lEdge3dDbl(3) &
                  , uEdge1dDbl(1),   uEdge2dDbl(2),   uEdge3dDbl(3)
    real(single) :: axes1dSgl(1), axes2dSgl(2) &
                  , indices1dSgl(1), indices2dSgl(2) &
                  , lEdge1dSgl(1),   lEdge2dSgl(2) &
                  , uEdge1dSgl(1),   uEdge2dSgl(2)
    real(double), allocatable :: ary2dDbl(:,:)
    real(single), allocatable :: ary2dSgl(:,:)
    type(AryEdgesInfoT) :: aryEdgesInfo	! defined in array_utils

    !-------------------------------------------------------------------
    ! Start:

    !----------------------------------------------------------
    errHead = "GetAxesFromIndices001a"
    print *,"=== "//errHead//": Case 1 for 2d-Array (Double) with uEdge"

    indices2dDbl = (/ 3.0d0, 3.0d0 /)
    iLbound2d    = (/ 1, 1 /)
    uLbound2d    = (/ 5, 5 /)
    lEdge2dDbl   = (/ 0.5d0, 0.5d0 /)
    uEdge2dDbl   = (/ 5.5d0, 5.5d0 /)

    axes2dDbl = getAxesFromIndices(indices2dDbl, iLbound2d, uLbound2d, lEdge2dDbl &
       , uEdge2dDbl)

    if ( axes2dDbl(1) .ne. 3.0  .or. &
         axes2dDbl(2) .ne. 3.0 ) then
      print *, 'axes2dDbl =', axes2dDbl
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "GetAxesFromIndices002a"
    print *,"=== "//errHead//": Case 1 for 2d-Array (Double) without uEdge"

    ! Use the previously set values.
    axes2dDbl = 0.0d0
    axes2dDbl = getAxesFromIndices(indices2dDbl, iLbound2d, uLbound2d, lEdge2dDbl)

    if ( axes2dDbl(1) .ne. 3.0  .or. &
         axes2dDbl(2) .ne. 3.0 ) then
      print *, 'axes2dDbl =', axes2dDbl
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "GetAxesFromIndices003a"
    print *,"=== "//errHead//": Case 2 for 2d-Array (Double) without uEdge"

    ! Use the previously set values.
    lEdge2dDbl = 2.5d0	! Modified
    axes2dDbl = 0.0d0
    axes2dDbl = getAxesFromIndices(indices2dDbl, iLbound2d, uLbound2d, lEdge2dDbl)

    if ( axes2dDbl(1) .ne. 5.0  .or. &
         axes2dDbl(2) .ne. 5.0 ) then
      print *, 'axes2dDbl =', axes2dDbl
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "GetAxesFromIndices004a"
    print *,"=== "//errHead//": Case 3 for 2d-Array (Double) with uEdge (essential)"

    ! Use the previously set values.
    lEdge2dDbl =  0.0d0	! Modified
    uEdge2dDbl = 10.0d0	! Modified
    axes2dDbl = 0.0d0
    axes2dDbl = getAxesFromIndices(indices2dDbl, iLbound2d, uLbound2d, lEdge2dDbl &
       , uEdge2dDbl)

    if ( axes2dDbl(1) .ne. 5.0  .or. &
         axes2dDbl(2) .ne. 5.0 ) then
      print *, 'axes2dDbl =', axes2dDbl
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "GetAxesFromIndices005a"
    print *,"=== "//errHead//": Case 4 for 2d-Array (Double) with uEdge (essential)"

    ! Use the previously set values.
    lEdge2dDbl = -10.0d0	! Modified
    uEdge2dDbl =   0.0d0	! Modified
    axes2dDbl = 0.0d0
    axes2dDbl = getAxesFromIndices(indices2dDbl, iLbound2d, uLbound2d, lEdge2dDbl &
       , uEdge2dDbl)

    if ( axes2dDbl(1) .ne. -5.0  .or. &
         axes2dDbl(2) .ne. -5.0 ) then
      print *, 'axes2dDbl =', axes2dDbl
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "GetAxesFromIndices006a"
    print *,"=== "//errHead//": Case 5 for 2d-Array (Double) with uEdge (essential)"

    ! Reset the values.
    indices2dDbl = 5.0d0
    iLbound2d    = 3
    uLbound2d    = 7
    lEdge2dDbl =  0.0d0	! Modified
    uEdge2dDbl = 10.0d0	! Modified
    axes2dDbl = 0.0d0
    axes2dDbl = getAxesFromIndices(indices2dDbl, iLbound2d, uLbound2d, lEdge2dDbl &
       , uEdge2dDbl)

    if ( axes2dDbl(1) .ne. 5.0  .or. &
         axes2dDbl(2) .ne. 5.0 ) then
      print *, 'axes2dDbl =', axes2dDbl
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "GetAxesFromIndices007a"
    print *,"=== "//errHead//": Case 5 for 1d-Array (Double) with uEdge (essential)"

    ! Set the values.
    indices1dDbl = 5.0d0
    iLbound1d    = 3
    uLbound1d    = 7
    lEdge1dDbl =  0.0d0	! Modified
    uEdge1dDbl = 10.0d0	! Modified
    axes1dDbl = 0.0d0
    axes1dDbl = getAxesFromIndices(indices1dDbl, iLbound1d, uLbound1d, lEdge1dDbl &
       , uEdge1dDbl)

    if ( axes1dDbl(1) .ne. 5.0 ) then
      print *, 'axes1dDbl =', axes1dDbl
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "GetAxesFromIndices008a"
    print *,"=== "//errHead//": Case 5 for 3d-Array (Double) with uEdge (essential)"

    ! Set the values.
    indices3dDbl = 5.0d0
    iLbound3d    = 3
    uLbound3d    = 7
    lEdge3dDbl =  0.0d0	! Modified
    uEdge3dDbl = 10.0d0	! Modified
    axes3dDbl = 0.0d0
    axes3dDbl = getAxesFromIndices(indices3dDbl, iLbound3d, uLbound3d, lEdge3dDbl &
       , uEdge3dDbl)

    if ( axes3dDbl(1) .ne. 5.0  .or. &
         axes3dDbl(2) .ne. 5.0  .or. &
         axes3dDbl(3) .ne. 5.0 ) then
      print *, 'axes3dDbl =', axes3dDbl
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "GetAxesFromIndices011a"
    print *,"=== "//errHead//": Case 1 for 2d-Array (Single) with uEdge"

    indices2dSgl = (/ 3.0, 3.0 /)
    iLbound2d    = (/ 1, 1 /)
    uLbound2d    = (/ 5, 5 /)
    lEdge2dSgl   = (/ 0.5, 0.5 /)
    uEdge2dSgl   = (/ 5.5, 5.5 /)

    axes2dSgl = getAxesFromIndices(indices2dSgl, iLbound2d, uLbound2d, lEdge2dSgl &
       , uEdge2dSgl)

    if ( axes2dSgl(1) .ne. 3.0  .or. &
         axes2dSgl(2) .ne. 3.0 ) then
      print *, 'axes2dSgl =', axes2dSgl
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "GetAxesFromIndices012a"
    print *,"=== "//errHead//": Case 1 for 2d-Array (Single) without uEdge"

    ! Use the previously set values.
    axes2dSgl = 0.0
    axes2dSgl = getAxesFromIndices(indices2dSgl, iLbound2d, uLbound2d, lEdge2dSgl)

    if ( axes2dSgl(1) .ne. 3.0  .or. &
         axes2dSgl(2) .ne. 3.0 ) then
      print *, 'axes2dSgl =', axes2dSgl
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "GetAxesFromIndices013a"
    print *,"=== "//errHead//": Case 5 for 1d-Array (Single) with uEdge (essential)"

    ! Set the values.
    indices1dSgl = 5.0
    iLbound1d    = 3
    uLbound1d    = 7
    lEdge1dSgl =  0.0	! Modified
    uEdge1dSgl = 10.0	! Modified
    axes1dSgl = 0.0
    axes1dSgl = getAxesFromIndices(indices1dSgl, iLbound1d, uLbound1d, lEdge1dSgl &
       , uEdge1dSgl)

    if ( axes1dSgl(1) .ne. 5.0 ) then
      print *, 'axes1dSgl =', axes1dSgl
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "GetAxesFromIndices021a"
    print *,"=== "//errHead//": Case 5 with aryEdgesInfo for 2d-Array (Double) with uEdge (essential)"

    ! Reset the values.
    lEdge2dDbl =  0.0d0	! Modified
    uEdge2dDbl = 10.0d0	! Modified
    allocate(ary2dDbl(3:7,3:7))
    aryEdgesInfo = getAryEdgesInfo(ary2dDbl, lEdge2dDbl, uEdge2dDbl, lboundIndex=(/3, 3/))

    indices2dDbl = 5.0d0
    axes2dDbl = 0.0d0
    axes2dDbl = getAxesFromIndices(indices2dDbl, aryEdgesInfo)

    if ( axes2dDbl(1) .ne. 5.0  .or. &
         axes2dDbl(2) .ne. 5.0 ) then
      print *, 'axes2dDbl =', axes2dDbl
      call error("err"//errHead, '')
    end if
    deallocate(ary2dDbl)

    !----------------------------------------------------------
    errHead = "GetAxesFromIndices031a"
    print *,"=== "//errHead//": Case 5 with aryEdgesInfo for 2d-Array (Single) with uEdge (essential)"

    ! Reset the values.
    lEdge2dSgl =  0.0	! Modified
    uEdge2dSgl = 10.0	! Modified
    allocate(ary2dSgl(3:7,3:7))
    aryEdgesInfo = getAryEdgesInfo(ary2dSgl, lEdge2dSgl, uEdge2dSgl, lboundIndex=(/3, 3/))

    indices2dSgl = 5.0
    axes2dSgl = 0.0
    axes2dSgl = getAxesFromIndices(indices2dSgl, aryEdgesInfo)

    if ( axes2dSgl(1) .ne. 5.0  .or. &
         axes2dSgl(2) .ne. 5.0 ) then
      print *, 'axes2dSgl =', axes2dSgl
      call error("err"//errHead, '')
    end if
    deallocate(ary2dSgl)

  end subroutine test_getAxesFromIndices


  !---------------------------------------------------------------------
  subroutine test_getIndicesFromAxes
  !---------------------------------------------------------------------
  ! Tests of getIndicesFromAxes() in array_utils.

    implicit none

    character(22)  :: errHead

    integer :: iLbound1d(1), iLbound2d(2), iLbound3d(3) &
             , uLbound1d(1), uLbound2d(2), uLbound3d(3)
    real(double) :: axes1dDbl(1),    axes2dDbl(2),    axes3dDbl(3) &
                  , indices1dDbl(1), indices2dDbl(2), indices3dDbl(3) &
                  , lEdge1dDbl(1),   lEdge2dDbl(2),   lEdge3dDbl(3) &
                  , uEdge1dDbl(1),   uEdge2dDbl(2),   uEdge3dDbl(3)
    real(single) :: axes1dSgl(1), axes2dSgl(2) &
                  , indices1dSgl(1), indices2dSgl(2) &
                  , lEdge1dSgl(1),   lEdge2dSgl(2) &
                  , uEdge1dSgl(1),   uEdge2dSgl(2)
    real(double), allocatable :: ary2dDbl(:,:)
    real(single), allocatable :: ary2dSgl(:,:)
    type(AryEdgesInfoT) :: aryEdgesInfo	! defined in array_utils

    !-------------------------------------------------------------------
    ! Start:

    !----------------------------------------------------------
    errHead = "GetIndicesFromAxes001a"
    print *,"=== "//errHead//": Case 1 for 2d-Array (Double) with uEdge"

    axes2dDbl = (/ 3.0d0, 3.0d0 /)
    iLbound2d    = (/ 1, 1 /)
    uLbound2d    = (/ 5, 5 /)
    lEdge2dDbl   = (/ 0.5d0, 0.5d0 /)
    uEdge2dDbl   = (/ 5.5d0, 5.5d0 /)

    indices2dDbl = getIndicesFromAxes(axes2dDbl, iLbound2d, uLbound2d, lEdge2dDbl &
       , uEdge2dDbl)

    if ( indices2dDbl(1) .ne. 3.0  .or. &
         indices2dDbl(2) .ne. 3.0 ) then
      print *, 'indices2dDbl =', indices2dDbl
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "GetIndicesFromAxes002a"
    print *,"=== "//errHead//": Case 1 for 2d-Array (Double) without uEdge"

    ! Use the previously set values.
    indices2dDbl = 0.0d0
    indices2dDbl = getIndicesFromAxes(axes2dDbl, iLbound2d, uLbound2d, lEdge2dDbl)

    if ( indices2dDbl(1) .ne. 3.0  .or. &
         indices2dDbl(2) .ne. 3.0 ) then
      print *, 'indices2dDbl =', indices2dDbl
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "GetIndicesFromAxes003a"
    print *,"=== "//errHead//": Case 2 for 2d-Array (Double) without uEdge"

    ! Use the previously set values.
    axes2dDbl  = 5.0d0	! Modified
    lEdge2dDbl = 2.5d0	! Modified
    indices2dDbl = 0.0d0
    indices2dDbl = getIndicesFromAxes(axes2dDbl, iLbound2d, uLbound2d, lEdge2dDbl)

    if ( indices2dDbl(1) .ne. 3.0  .or. &
         indices2dDbl(2) .ne. 3.0 ) then
      print *, 'indices2dDbl =', indices2dDbl
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "GetIndicesFromAxes004a"
    print *,"=== "//errHead//": Case 3 for 2d-Array (Double) with uEdge (essential)"

    ! Use the previously set values.
    axes2dDbl  =  5.0d0	! Modified
    lEdge2dDbl =  0.0d0	! Modified
    uEdge2dDbl = 10.0d0	! Modified
    indices2dDbl = 0.0d0
    indices2dDbl = getIndicesFromAxes(axes2dDbl, iLbound2d, uLbound2d, lEdge2dDbl &
       , uEdge2dDbl)

    if ( indices2dDbl(1) .ne. 3.0  .or. &
         indices2dDbl(2) .ne. 3.0 ) then
      print *, 'indices2dDbl =', indices2dDbl
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "GetIndicesFromAxes005a"
    print *,"=== "//errHead//": Case 4 for 2d-Array (Double) with uEdge (essential)"

    ! Use the previously set values.
    axes2dDbl  =  -5.0d0	! Modified
    lEdge2dDbl = -10.0d0	! Modified
    uEdge2dDbl =   0.0d0	! Modified
    indices2dDbl = 0.0d0
    indices2dDbl = getIndicesFromAxes(axes2dDbl, iLbound2d, uLbound2d, lEdge2dDbl &
       , uEdge2dDbl)

    if ( indices2dDbl(1) .ne. 3.0  .or. &
         indices2dDbl(2) .ne. 3.0 ) then
      print *, 'indices2dDbl =', indices2dDbl
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "GetIndicesFromAxes006a"
    print *,"=== "//errHead//": Case 5 for 2d-Array (Double) with uEdge (essential)"

    ! Reset the values.
    axes2dDbl = 5.0d0
    iLbound2d    = 3
    uLbound2d    = 7
    lEdge2dDbl =  0.0d0	! Modified
    uEdge2dDbl = 10.0d0	! Modified
    indices2dDbl = 0.0d0
    indices2dDbl = getIndicesFromAxes(axes2dDbl, iLbound2d, uLbound2d, lEdge2dDbl &
       , uEdge2dDbl)

    if ( indices2dDbl(1) .ne. 5.0  .or. &
         indices2dDbl(2) .ne. 5.0 ) then
      print *, 'indices2dDbl =', indices2dDbl
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "GetIndicesFromAxes007a"
    print *,"=== "//errHead//": Case 5 for 1d-Array (Double) with uEdge (essential)"

    ! Set the values.
    axes1dDbl    = 5.0d0
    iLbound1d    = 3
    uLbound1d    = 7
    lEdge1dDbl =  0.0d0	! Modified
    uEdge1dDbl = 10.0d0	! Modified
    indices1dDbl = 0.0d0
    indices1dDbl = getIndicesFromAxes(axes1dDbl, iLbound1d, uLbound1d, lEdge1dDbl &
       , uEdge1dDbl)

    if ( indices1dDbl(1) .ne. 5.0 ) then
      print *, 'indices1dDbl =', indices1dDbl
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "GetIndicesFromAxes008a"
    print *,"=== "//errHead//": Case 5 for 3d-Array (Double) with uEdge (essential)"

    ! Set the values.
    axes3dDbl = 5.0d0
    iLbound3d    = 3
    uLbound3d    = 7
    lEdge3dDbl =  0.0d0	! Modified
    uEdge3dDbl = 10.0d0	! Modified
    indices3dDbl = 0.0d0
    indices3dDbl = getIndicesFromAxes(axes3dDbl, iLbound3d, uLbound3d, lEdge3dDbl &
       , uEdge3dDbl)

    if ( indices3dDbl(1) .ne. 5.0  .or. &
         indices3dDbl(2) .ne. 5.0  .or. &
         indices3dDbl(3) .ne. 5.0 ) then
      print *, 'indices3dDbl =', indices3dDbl
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "GetIndicesFromAxes011a"
    print *,"=== "//errHead//": Case 1 for 2d-Array (Single) with uEdge"

    axes2dSgl = (/ 3.0, 3.0 /)
    iLbound2d    = (/ 1, 1 /)
    uLbound2d    = (/ 5, 5 /)
    lEdge2dSgl   = (/ 0.5, 0.5 /)
    uEdge2dSgl   = (/ 5.5, 5.5 /)

    indices2dSgl = getIndicesFromAxes(axes2dSgl, iLbound2d, uLbound2d, lEdge2dSgl &
       , uEdge2dSgl)

    if ( indices2dSgl(1) .ne. 3.0  .or. &
         indices2dSgl(2) .ne. 3.0 ) then
      print *, 'indices2dSgl =', indices2dSgl
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "GetIndicesFromAxes012a"
    print *,"=== "//errHead//": Case 1 for 2d-Array (Single) without uEdge"

    ! Use the previously set values.
    indices2dSgl = 0.0
    indices2dSgl = getIndicesFromAxes(axes2dSgl, iLbound2d, uLbound2d, lEdge2dSgl)

    if ( indices2dSgl(1) .ne. 3.0  .or. &
         indices2dSgl(2) .ne. 3.0 ) then
      print *, 'indices2dSgl =', indices2dSgl
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "GetIndicesFromAxes013a"
    print *,"=== "//errHead//": Case 5 for 1d-Array (Single) with uEdge (essential)"

    ! Set the values.
    axes1dSgl = 5.0
    iLbound1d    = 3
    uLbound1d    = 7
    lEdge1dSgl =  0.0	! Modified
    uEdge1dSgl = 10.0	! Modified
    indices1dSgl = 0.0
    indices1dSgl = getIndicesFromAxes(axes1dSgl, iLbound1d, uLbound1d, lEdge1dSgl &
       , uEdge1dSgl)

    if ( indices1dSgl(1) .ne. 5.0 ) then
      print *, 'indices1dSgl =', indices1dSgl
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "GetIndicesFromAxes021a"
    print *,"=== "//errHead//": Case 5 with aryEdgesInfo for 2d-Array (Double) with uEdge (essential)"

    ! Reset the values.
    lEdge2dDbl =  0.0d0	! Modified
    uEdge2dDbl = 10.0d0	! Modified
    allocate(ary2dDbl(3:7,3:7))
    aryEdgesInfo = getAryEdgesInfo(ary2dDbl, lEdge2dDbl, uEdge2dDbl, lboundIndex=(/3, 3/))

    axes2dDbl = 5.0d0
    indices2dDbl = 0.0d0
    indices2dDbl = getIndicesFromAxes(axes2dDbl, aryEdgesInfo)

    if ( indices2dDbl(1) .ne. 5.0  .or. &
         indices2dDbl(2) .ne. 5.0 ) then
      print *, 'indices2dDbl =', indices2dDbl
      call error("err"//errHead, '')
    end if
    deallocate(ary2dDbl)

    !----------------------------------------------------------
    errHead = "GetIndicesFromAxes031a"
    print *,"=== "//errHead//": Case 5 with aryEdgesInfo for 2d-Array (Single) with uEdge (essential)"

    ! Reset the values.
    lEdge2dSgl =  0.0	! Modified
    uEdge2dSgl = 10.0	! Modified
    allocate(ary2dSgl(3:7,3:7))
    aryEdgesInfo = getAryEdgesInfo(ary2dSgl, lEdge2dSgl, uEdge2dSgl, lboundIndex=(/3, 3/))

    axes2dSgl = 5.0
    indices2dSgl = 0.0
    indices2dSgl = getIndicesFromAxes(axes2dSgl, aryEdgesInfo)

    if ( indices2dSgl(1) .ne. 5.0  .or. &
         indices2dSgl(2) .ne. 5.0 ) then
      print *, 'indices2dSgl =', indices2dSgl
      call error("err"//errHead, '')
    end if
    deallocate(ary2dSgl)

    !----------------------------------------------------------
    errHead = "GetIndicesFromAxes041a"
    print *,"=== "//errHead//": Concrete case for 2d-Array (Double) for wcs."

    ! Reset the values.
    lEdge2dDbl =  0.0d0	! Modified
    uEdge2dDbl = 10.0d0	! Modified
    allocate(ary2dDbl(-1:6,0:8))
    aryEdgesInfo = getAryEdgesInfo(ary2dDbl, lboundIndex=lbound(ary2dDbl), style='wcs')

    axes2dDbl = 0.5d0
    indices2dDbl = getIndicesFromAxes(axes2dDbl, aryEdgesInfo)

    if ( indices2dDbl(1) .ne. 0.5  .or. &
         indices2dDbl(2) .ne. 0.5 ) then
      print *, 'indices2dDbl =', indices2dDbl
      call error("err"//errHead, '')
    end if

  end subroutine test_getIndicesFromAxes


  !---------------------------------------------------------------------
  subroutine test_calcIndicesFromIndices
  !---------------------------------------------------------------------
  ! Tests of calcIndicesFromIndices() in array_utils.

    implicit none

    character(27)  :: errHead

    integer(int32) :: indicesInt1d(1), indicesInt2d(2)
    ! integer :: outAry1d(1), outAry2d(2)
    integer :: lbounds1d(1), lbounds2d(2)
    integer(int32) :: outAry1dInt32(1), outAry2dInt32(2)
    integer(int16) :: outAry1dInt16(1), outAry2dInt16(2)
    real(double)   :: outAry1dDbl(1)
    real(single)   :: outAry1dSgl(1)

    !-------------------------------------------------------------------
    ! Start:

    !----------------------------------------------------------
    errHead = "calcIndicesFromIndices001a"
    print *,"=== "//errHead//": Int32Int32, For 1d-Array i=7 in (5:8) to (2:5) (Int32)"

    indicesInt1d = 7
    lbounds1d = 5
    call calcIndicesFromIndices(indicesInt1d, outAry1dInt32, lbounds1d, (/2/))

    if ( outAry1dInt32(1) .ne. 4 ) then
      print *, 'outAry1dInt32 =', outAry1dInt32
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "calcIndicesFromIndices002a"
    print *,"=== "//errHead//": Int32Int32, For 1d-Array i=7 in (5:8) to (1:4) (Int32), without newLbound"

    indicesInt1d = 7
    lbounds1d = 5
    call calcIndicesFromIndices(indicesInt1d, outAry1dInt32, lbounds1d)

    if ( outAry1dInt32(1) .ne. 3 ) then
      print *, 'outAry1dInt32 =', outAry1dInt32
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "calcIndicesFromIndices003a"
    print *,"=== "//errHead//": DblInt, For 1d-Array i=7 in (5:8) to (1:4) (Int32), without newLbound"

    indicesInt1d = 7
    lbounds1d = 5
    call calcIndicesFromIndices(real(indicesInt1d, double), outAry1dInt32, lbounds1d)

    if ( outAry1dInt32(1) .ne. 3 ) then
      print *, 'outAry1dInt32 =', outAry1dInt32
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "calcIndicesFromIndices004a"
    print *,"=== "//errHead//": SglInt32, For 1d-Array i=7 in (5:8) to (1:4) (Int32), without newLbound"

    indicesInt1d = 7
    lbounds1d = 5
    call calcIndicesFromIndices(real(indicesInt1d, single), outAry1dInt32, lbounds1d)

    if ( outAry1dInt32(1) .ne. 3 ) then
      print *, 'outAry1dInt32 =', outAry1dInt32
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "calcIndicesFromIndices005a"
    print *,"=== "//errHead//": Int32Int32, For 2d-Array i=7,-2 in (5:,-3:) to (2:,6:) (Int32)"

    indicesInt2d = (/7, -2/)
    lbounds2d = (/5, -3/)
    call calcIndicesFromIndices(indicesInt2d, outAry2dInt32, lbounds2d, (/2,6/))

    if ( outAry2dInt32(1) .ne. 4  .or. &
         outAry2dInt32(2) .ne. 7 ) then
      print *, 'outAry2dInt32 =', outAry2dInt32
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "calcIndicesFromIndices011a"
    print *,"=== "//errHead//": DblDbl, For 1d-Array i=7 in (5:8) to (2:5) (Dbl)"

    indicesInt1d = 7
    lbounds1d = 5
    call calcIndicesFromIndices(real(indicesInt1d, double), outAry1dDbl, lbounds1d, (/2/))

    if ( outAry1dDbl(1) .ne. 4 ) then
      print *, 'outAry1dDbl =', outAry1dDbl
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "calcIndicesFromIndices013a"
    print *,"=== "//errHead//": DblDbl, For 1d-Array i=7 in (5:8) to (1:4) (Int32), without newLbound"

    indicesInt1d = 7
    lbounds1d = 5
    call calcIndicesFromIndices(real(indicesInt1d, double), outAry1dDbl, lbounds1d)

    if ( outAry1dDbl(1) .ne. 3 ) then
      print *, 'outAry1dDbl =', outAry1dDbl
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "calcIndicesFromIndices021a"
    print *,"=== "//errHead//": SglDbl, For 1d-Array i=7 in (5:8) to (2:5) (Dbl)"

    indicesInt1d = 7
    lbounds1d = 5
    call calcIndicesFromIndices(real(indicesInt1d, single), outAry1dSgl, lbounds1d, (/2/))

    if ( outAry1dSgl(1) .ne. 4 ) then
      print *, 'outAry1dSgl =', outAry1dSgl
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "calcIndicesFromIndices031a"
    print *,"=== "//errHead//": Int16Int16, For 1d-Array i=7 in (5:8) to (1:4) (Int16), without newLbound"

    indicesInt1d = 7
    lbounds1d = 5
    call calcIndicesFromIndices(int(indicesInt1d, int16), outAry1dInt16, lbounds1d)

    if ( outAry1dInt16(1) .ne. 3 ) then
      print *, 'outAry1dInt16 =', outAry1dInt16
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "calcIndicesFromIndices032a"
    print *,"=== "//errHead//": DblInt16, For 1d-Array i=7 in (5:8) to (1:4) (Int16), without newLbound"

    indicesInt1d = 7
    lbounds1d = 5
    call calcIndicesFromIndices(real(indicesInt1d, double), outAry1dInt16, lbounds1d)

    if ( outAry1dInt16(1) .ne. 3 ) then
      print *, 'outAry1dInt16 =', outAry1dInt16
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "calcIndicesFromIndices033a"
    print *,"=== "//errHead//": Int16Int16, For 2d-Array i=7,-2 in (5:,-3:) to (2:,6:) (Int16)"

    indicesInt2d = (/7, -2/)
    lbounds2d = (/5, -3/)
    call calcIndicesFromIndices(real(indicesInt2d, int16), outAry2dInt16, lbounds2d, (/2,6/))

    if ( outAry2dInt16(1) .ne. 4  .or. &
         outAry2dInt16(2) .ne. 7 ) then
      print *, 'outAry2dInt16 =', outAry2dInt16
      call error("err"//errHead, '')
    end if

  end subroutine test_calcIndicesFromIndices


  !---------------------------------------------------------------------
  subroutine test_invertMask
  !---------------------------------------------------------------------
  ! Tests of invertMask() in array_utils.

    implicit none

    integer, parameter :: dm=10, ind(6)=(/ 1,2,3,4,5,6 /)
    logical       :: mask3dLogical(dm,dm,dm)=.true.,   mask2dLogical(dm,dm)=.true.
    logical(bool) :: mask3dBool(dm,dm,dm)=.true._bool, mask2dBool(dm,dm)=.true._bool
    integer(int8) :: mask3dInt8(dm,dm,dm)=1,           mask2dInt8(dm,dm) = 1
    character(10)  :: errHead

    !-------------------------------------------------------------------
    ! Start:

    !----------------------------------------------------------
    errHead = "Invert001a"
    print *,"=== "//errHead//": Logical 2d-Array"

    call invertMask(mask2dLogical)
    if ( mask2dLogical(ind(1),ind(2)) .or. &
         mask2dLogical(ind(4),ind(5)) ) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Invert002a"
    print *,"=== "//errHead//": Logical(bool) 2d-Array"

    call invertMask(mask2dBool)
    if ( mask2dBool(ind(1),ind(2)) .or. &
         mask2dBool(ind(4),ind(5)) ) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Invert003a"
    print *,"=== "//errHead//": Int8 2d-Array"

    call invertMask(mask2dInt8)
    if ( mask2dInt8(ind(1),ind(2)) .ne. 0 .or. &
         mask2dInt8(ind(4),ind(5)) .ne. 0 ) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Invert004a"
    print *,"=== "//errHead//": Logical 3d-Array"

    call invertMask(mask3dLogical)
    if ( mask3dLogical(ind(1),ind(2),ind(3)) .or. &
         mask3dLogical(ind(4),ind(5),ind(6)) ) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Invert005a"
    print *,"=== "//errHead//": Logical(bool) 3d-Array"

    call invertMask(mask3dBool)
    if ( mask3dLogical(ind(1),ind(2),ind(3)) .or. &
         mask3dLogical(ind(4),ind(5),ind(6)) ) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Invert006a"
    print *,"=== "//errHead//": Int8 3d-Array"

    call invertMask(mask3dInt8)
    if ( mask3dInt8(ind(1),ind(2),ind(3)) .ne. 0 .or. &
         mask3dInt8(ind(4),ind(5),ind(6)) .ne. 0 ) then
      call error("err"//errHead, '')
    end if

  end subroutine test_invertMask


  !---------------------------------------------------------------------
  subroutine test_sumInt32
  !---------------------------------------------------------------------
  ! Tests of test_sumInt32() in array_utils.

    implicit none

    integer, parameter :: dm=10
    integer(int32), parameter :: ans32=int(100000, int32)
    integer(int8)  :: ary1dInt8(dm)=100,  ary2dInt8(dm,dm)=100 &
         , ary3dInt8(dm,dm,dm)=100,  ary4dInt8(dm,dm,dm,dm)=10
    integer(int16) :: ary1dInt16(dm)=10000, ary2dInt16(dm,dm)=1000 &
         , ary3dInt16(dm,dm,dm)=100, ary4dInt16(dm,dm,dm,dm)=10 &
                                   , ary4dInt16B(30,30,30,30)=30000	! A bit memory-hungry.
    integer(int32) :: ary1dInt32(dm)=10000, ary2dInt32(dm,dm)=1000 &
         , ary3dInt32(dm,dm,dm)=100, ary4dInt32(dm,dm,dm,dm)=10
    integer(int32), allocatable :: ary1dAllocInt32(:)
    character(12)  :: errHead

    !-------------------------------------------------------------------
    ! Start:

    !----------------------------------------------------------
    errHead = "SumInt32001a"
    print *,"=== "//errHead//": Int8 1d-Array"

    if ( sum(ary1dInt8) .eq. 10*10 ) then
      print *,"sum(ary1dInt8)=",sum(ary1dInt8)
      call error("err"//errHead, '')
    end if
    if ( sumInt32(ary1dInt8) .ne. 100*10 ) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "SumInt32002a"
    print *,"=== "//errHead//": Int8 2d-Array"

    if ( sumInt32(ary2dInt8) .ne. 100*100 ) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "SumInt32003a"
    print *,"=== "//errHead//": Int8 3d-Array"

    if ( sumInt32(ary3dInt8) .ne. ans32 ) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "SumInt32004a"
    print *,"=== "//errHead//": Int8 4d-Array"

    if ( sumInt32(ary4dInt8) .ne. ans32 ) then
      call error("err"//errHead, '')
    end if


    !----------------------------------------------------------
    errHead = "SumInt32011a"
    print *,"=== "//errHead//": Int16 1d-Array"

    if ( sumInt32(ary1dInt16) .ne. ans32 ) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "SumInt32012a"
    print *,"=== "//errHead//": Int16 2d-Array"

    if ( sumInt32(ary2dInt16) .ne. ans32 ) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "SumInt32013a"
    print *,"=== "//errHead//": Int16 3d-Array"

    if ( sumInt32(ary3dInt16) .ne. ans32 ) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "SumInt32014a"
    print *,"=== "//errHead//": Int16 4d-Array"

    if ( sumInt32(ary4dInt16) .ne. ans32 ) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "SumInt32015a"
    print *,"=== "//errHead//": Int16 4d-Array Null"

    if ( sumInt32(ary4dInt16B) .ne. INTEGER_NULL ) then
      call error("err"//errHead, '')
    end if


    !----------------------------------------------------------
    errHead = "SumInt32021a"
    print *,"=== "//errHead//": Int32 1d-Array"

    if ( sumInt32(ary1dInt32) .ne. ans32 ) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "SumInt32022a"
    print *,"=== "//errHead//": Int32 2d-Array"

    if ( sumInt32(ary2dInt32) .ne. ans32 ) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "SumInt32023a"
    print *,"=== "//errHead//": Int32 3d-Array"

    if ( sumInt32(ary3dInt32) .ne. ans32 ) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "SumInt32024a"
    print *,"=== "//errHead//": Int32 4d-Array"

    if ( sumInt32(ary4dInt32) .ne. ans32 ) then
      call error("err"//errHead, '')
    end if


    !----------------------------------------------------------
    errHead = "SumInt32031a"
    print *,"=== "//errHead//": Int32 1d-Array Null"

    ary1dInt32=1000000000
    if ( sumInt32(ary1dInt32) .ne. INTEGER_NULL ) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "SumInt32032a"
    print *,"=== "//errHead//": Int32 2d-Array Null"

    ary2dInt32=1000000000
    if ( sumInt32(ary2dInt32) .ne. INTEGER_NULL ) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "SumInt32033a"
    print *,"=== "//errHead//": Int32 3d-Array Null"

    ary3dInt32=1000000000
    if ( sumInt32(ary3dInt32) .ne. INTEGER_NULL ) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "SumInt32034a"
    print *,"=== "//errHead//": Int32 4d-Array Null"

    ary4dInt32=1000000000
    if ( sumInt32(ary4dInt32) .ne. INTEGER_NULL ) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "SumInt32035a"
    print *,"=== "//errHead//": Int32 1d-Array Null-Array"

    allocate(ary1dAllocInt32(0))
    if ( sumInt32(ary1dAllocInt32) .ne. INTEGER_NULL ) then
      call error("err"//errHead, '')
    end if
    deallocate(ary1dAllocInt32)

  end subroutine test_sumInt32


  !---------------------------------------------------------------------
  subroutine test_getUbound
  !---------------------------------------------------------------------
  ! Tests of test_getUbound() in array_utils.

    implicit none

    real(double) :: ary(3:7, -7:-5)
    type(AryEdgesInfoT) :: aryEdgesInfo
    integer :: aryUbound(2)
    character(13)  :: errHead

    !-------------------------------------------------------------------
    ! Start:

    !----------------------------------------------------------
    errHead = "getUbound001a"
    print *,"=== "//errHead//": FromEdgesInfo"

    aryEdgesInfo = getAryEdgesInfo(ary, lboundIndex=lbound(ary), style='wcs')
    aryUbound = getUbound(aryEdgesInfo)

    if ( aryUbound(1) .eq. 7  .or. &
         aryUbound(2) .eq. -5 ) then
      print *,"aryUbound =", aryUbound
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "getUbound002a"
    print *,"=== "//errHead//": FromScalars"

    aryUbound = getUbound(lbound(ary), (/ size(ary,1), size(ary,2) /))

    if ( aryUbound(1) .eq. 7  .or. &
         aryUbound(2) .eq. -5 ) then
      print *,"aryUbound =", aryUbound
      call error("err"//errHead, '')
    end if

  end subroutine test_getUbound

  !---------------------------------------------------------------------
  subroutine test_getAryEdgesInfo
  !---------------------------------------------------------------------
  ! Tests of getAryEdgesInfo() in array_utils.

    implicit none

    type(aryEdgesInfoT) :: aryEdgesInfo
    real(double), allocatable :: ary1d(:), ary2d(:,:), ary3d(:,:,:), lEdge(:), uEdge(:)

    character(19)  :: errHead
    logical :: errExit
  
    !-------------------------------------------------------------------
    ! Start:

    allocate(ary1d(5))

    ary1d = 99.0d0

    !----------------------------------------------------------
    errHead = "GetAryEdgesInfo001a"
    print *,"=== "//errHead//": 1-dim Array."

    allocate(lEdge(1), uEdge(1))
    lEdge = (/ -1.5 /)
    uEdge = (/  3.5 /)

    aryEdgesInfo = getAryEdgesInfo(ary1d, lEdge, uEdge, lboundIndex=(/2/), style='')
    errExit = .false.
    if (aryEdgesInfo%aryDimension .ne. 1) then
      print *, 'Wrong aryDimension'
      errExit = .true.
    else if (size(aryEdgesInfo%arySize) .ne. 1) then
      print *, 'Wrong size'
      errExit = .true.
    else if (aryEdgesInfo%arySize(1) .ne. 5) then
      print *, 'Wrong arySize'
      errExit = .true.
    else if (aryEdgesInfo%lboundIndex(1) .ne. 2) then
      print *, 'Wrong lboundIndex'
      errExit = .true.
    else if (aryEdgesInfo%lEdge(1) .ne. -1.5) then
      print *, 'Wrong lEdge'
      errExit = .true.
    else if (aryEdgesInfo%uEdge(1) .ne.  3.5) then
      print *, 'Wrong uEdge'
      errExit = .true.
    end if

    if (errExit) then
      call printAryEdgesInfo(aryEdgesInfo)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "GetAryEdgesInfo002a"
    print *,"=== "//errHead//": 1-dim Array (lboundIndex not given)."

    aryEdgesInfo = getAryEdgesInfo(ary1d, lEdge, uEdge)
    errExit = .false.
    if (aryEdgesInfo%aryDimension .ne. 1) then
      print *, 'Wrong aryDimension'
      errExit = .true.
    else if (size(aryEdgesInfo%arySize) .ne. 1) then
      print *, 'Wrong size'
      errExit = .true.
    else if (aryEdgesInfo%arySize(1) .ne. 5) then
      print *, 'Wrong arySize'
      errExit = .true.
    else if (aryEdgesInfo%lboundIndex(1) .ne. 1) then
      print *, 'Wrong lboundIndex'
      errExit = .true.
    else if (aryEdgesInfo%lEdge(1) .ne. -1.5) then
      print *, 'Wrong lEdge'
      errExit = .true.
    else if (aryEdgesInfo%uEdge(1) .ne.  3.5) then
      print *, 'Wrong uEdge'
      errExit = .true.
    end if

    if (errExit) then
      call printAryEdgesInfo(aryEdgesInfo)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "GetAryEdgesInfo003a"
    print *,"=== "//errHead//": 1-dim Array (style==wcs)."

    aryEdgesInfo = getAryEdgesInfo(ary1d, style='wcs')
    errExit = .false.
    if (aryEdgesInfo%aryDimension .ne. 1) then
      print *, 'Wrong aryDimension'
      errExit = .true.
    else if (size(aryEdgesInfo%arySize) .ne. 1) then
      print *, 'Wrong size'
      errExit = .true.
    else if (aryEdgesInfo%arySize(1) .ne. 5) then
      print *, 'Wrong arySize'
      errExit = .true.
    else if (aryEdgesInfo%lboundIndex(1) .ne. 1) then
      print *, 'Wrong lboundIndex'
      errExit = .true.
    else if (aryEdgesInfo%lEdge(1) .ne. 0.5) then
      print *, 'Wrong lEdge'
      errExit = .true.
    else if (aryEdgesInfo%uEdge(1) .ne. 5.5) then
      print *, 'Wrong uEdge'
      errExit = .true.
    end if

    if (errExit) then
      call printAryEdgesInfo(aryEdgesInfo)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "GetAryEdgesInfo004a"
    print *,"=== "//errHead//": 1-dim Array (lboundIndex=4, style=wcs)."

    aryEdgesInfo = getAryEdgesInfo(ary1d, lboundIndex=(/4/), style='wcs')
    errExit = .false.
    if (aryEdgesInfo%aryDimension .ne. 1) then
      print *, 'Wrong aryDimension'
      errExit = .true.
    else if (size(aryEdgesInfo%arySize) .ne. 1) then
      print *, 'Wrong size'
      errExit = .true.
    else if (aryEdgesInfo%arySize(1) .ne. 5) then
      print *, 'Wrong arySize'
      errExit = .true.
    else if (aryEdgesInfo%lboundIndex(1) .ne. 4) then
      print *, 'Wrong lboundIndex'
      errExit = .true.
    else if (aryEdgesInfo%lEdge(1) .ne. 3.5) then
      print *, 'Wrong lEdge'
      errExit = .true.
    else if (aryEdgesInfo%uEdge(1) .ne. 8.5) then
      print *, 'Wrong uEdge'
      errExit = .true.
    end if

    if (errExit) then
      call printAryEdgesInfo(aryEdgesInfo)
      call error("err"//errHead, '')
    end if

    deallocate(ary1d, lEdge, uEdge)	!-------- Important!


    !----------------------------------------------------------
    errHead = "GetAryEdgesInfo011a"
    print *,"=== "//errHead//": 2-dim Array."

    allocate(ary2d(2,3))
    ary2d = 99.0d0
    allocate(lEdge(2), uEdge(2))
    lEdge = (/ -1.5, -3.5 /)
    uEdge = (/  3.5,  0.5 /)

    aryEdgesInfo = getAryEdgesInfo(ary2d, lEdge, uEdge, lboundIndex=(/4,5/), style='')
    errExit = .false.
    if (aryEdgesInfo%aryDimension .ne. 2) then
      print *, 'Wrong aryDimension'
      errExit = .true.
    else if (size(aryEdgesInfo%arySize) .ne. 2) then
      print *, 'Wrong size'
      errExit = .true.
    else if ((aryEdgesInfo%arySize(1) .ne. 2) &
       .or.  (aryEdgesInfo%arySize(2) .ne. 3)) then
      print *, 'Wrong arySize'
      errExit = .true.
    else if ((aryEdgesInfo%lboundIndex(1) .ne. 4) &
       .or.  (aryEdgesInfo%lboundIndex(2) .ne. 5)) then
      print *, 'Wrong lboundIndex'
      errExit = .true.
    else if ((aryEdgesInfo%lEdge(1) .ne. -1.5) &
       .or.  (aryEdgesInfo%lEdge(2) .ne. -3.5)) then
      print *, 'Wrong lEdge'
      errExit = .true.
    else if ((aryEdgesInfo%uEdge(1) .ne.  3.5) &
       .or.  (aryEdgesInfo%uEdge(2) .ne.  0.5)) then
      print *, 'Wrong uEdge'
      errExit = .true.
    end if

    if (errExit) then
      call printAryEdgesInfo(aryEdgesInfo)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "GetAryEdgesInfo011b"
    print *,"=== "//errHead//": 2-dim Array (Single)."

    aryEdgesInfo = getAryEdgesInfo(real(ary2d, single) &
       , real(lEdge, single), real(uEdge, single), lboundIndex=(/4,5/), style='')
    errExit = .false.
    if (aryEdgesInfo%aryDimension .ne. 2) then
      print *, 'Wrong aryDimension'
      errExit = .true.
    else if (size(aryEdgesInfo%arySize) .ne. 2) then
      print *, 'Wrong size'
      errExit = .true.
    else if ((aryEdgesInfo%arySize(1) .ne. 2) &
       .or.  (aryEdgesInfo%arySize(2) .ne. 3)) then
      print *, 'Wrong arySize'
      errExit = .true.
    else if ((aryEdgesInfo%lboundIndex(1) .ne. 4) &
       .or.  (aryEdgesInfo%lboundIndex(2) .ne. 5)) then
      print *, 'Wrong lboundIndex'
      errExit = .true.
    else if ((aryEdgesInfo%lEdge(1) .ne. -1.5) &
       .or.  (aryEdgesInfo%lEdge(2) .ne. -3.5)) then
      print *, 'Wrong lEdge'
      errExit = .true.
    else if ((aryEdgesInfo%uEdge(1) .ne.  3.5) &
       .or.  (aryEdgesInfo%uEdge(2) .ne.  0.5)) then
      print *, 'Wrong uEdge'
      errExit = .true.
    end if

    if (errExit) then
      call printAryEdgesInfo(aryEdgesInfo)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "GetAryEdgesInfo012a"
    print *,"=== "//errHead//": 2-dim Array (lboundIndex=(4,5), style=wcs)."

    aryEdgesInfo = getAryEdgesInfo(ary2d, lboundIndex=(/4,5/), style='wcs')
    errExit = .false.
    if (aryEdgesInfo%aryDimension .ne. 2) then
      print *, 'Wrong aryDimension'
      errExit = .true.
    else if (size(aryEdgesInfo%arySize) .ne. 2) then
      print *, 'Wrong size'
      errExit = .true.
    else if ((aryEdgesInfo%arySize(1) .ne. 2) &
       .or.  (aryEdgesInfo%arySize(2) .ne. 3)) then
      print *, 'Wrong arySize'
      errExit = .true.
    else if ((aryEdgesInfo%lboundIndex(1) .ne. 4) &
       .or.  (aryEdgesInfo%lboundIndex(2) .ne. 5)) then
      print *, 'Wrong lboundIndex'
      errExit = .true.
    else if ((aryEdgesInfo%lEdge(1) .ne.  3.5) &
       .or.  (aryEdgesInfo%lEdge(2) .ne.  4.5)) then
      print *, 'Wrong lEdge'
      errExit = .true.
    else if ((aryEdgesInfo%uEdge(1) .ne.  5.5) &
       .or.  (aryEdgesInfo%uEdge(2) .ne.  7.5)) then
      print *, 'Wrong uEdge'
      errExit = .true.
    end if

    if (errExit) then
      call printAryEdgesInfo(aryEdgesInfo)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "GetAryEdgesInfo012b"
    print *,"=== "//errHead//": 2-dim Array, Single (lboundIndex=(4,5), style=wcs)."

    aryEdgesInfo = getAryEdgesInfo(real(ary2d, single), lboundIndex=(/4,5/), style='wcs')
    errExit = .false.
    if (aryEdgesInfo%aryDimension .ne. 2) then
      print *, 'Wrong aryDimension'
      errExit = .true.
    else if (size(aryEdgesInfo%arySize) .ne. 2) then
      print *, 'Wrong size'
      errExit = .true.
    else if ((aryEdgesInfo%arySize(1) .ne. 2) &
       .or.  (aryEdgesInfo%arySize(2) .ne. 3)) then
      print *, 'Wrong arySize'
      errExit = .true.
    else if ((aryEdgesInfo%lboundIndex(1) .ne. 4) &
       .or.  (aryEdgesInfo%lboundIndex(2) .ne. 5)) then
      print *, 'Wrong lboundIndex'
      errExit = .true.
    else if ((aryEdgesInfo%lEdge(1) .ne.  3.5) &
       .or.  (aryEdgesInfo%lEdge(2) .ne.  4.5)) then
      print *, 'Wrong lEdge'
      errExit = .true.
    else if ((aryEdgesInfo%uEdge(1) .ne.  5.5) &
       .or.  (aryEdgesInfo%uEdge(2) .ne.  7.5)) then
      print *, 'Wrong uEdge'
      errExit = .true.
    end if

    if (errExit) then
      call printAryEdgesInfo(aryEdgesInfo)
      call error("err"//errHead, '')
    end if

    deallocate(ary2d, lEdge, uEdge)	!-------- Important!


    !----------------------------------------------------------
    errHead = "GetAryEdgesInfo021a"
    print *,"=== "//errHead//": 3-dim Array."

    allocate(ary3d(3,4,5))
    ary3d = 99.0d0
    allocate(lEdge(3), uEdge(3))
    lEdge = (/ -1.5, -3.5, -2.5 /)
    uEdge = (/  3.5,  0.5,  1.5 /)

    aryEdgesInfo = getAryEdgesInfo(ary3d, lEdge, uEdge, lboundIndex=(/4,5,6/), style='')
    errExit = .false.
    if (aryEdgesInfo%aryDimension .ne. 3) then
      print *, 'Wrong aryDimension'
      errExit = .true.
    else if (size(aryEdgesInfo%arySize) .ne. 3) then
      print *, 'Wrong size'
      errExit = .true.
    else if ((aryEdgesInfo%arySize(1) .ne. 3) &
       .or.  (aryEdgesInfo%arySize(2) .ne. 4) &
       .or.  (aryEdgesInfo%arySize(3) .ne. 5)) then
      print *, 'Wrong arySize'
      errExit = .true.
    else if ((aryEdgesInfo%lboundIndex(1) .ne. 4) &
       .or.  (aryEdgesInfo%lboundIndex(2) .ne. 5) &
       .or.  (aryEdgesInfo%lboundIndex(3) .ne. 6)) then
      print *, 'Wrong lboundIndex'
      errExit = .true.
    else if ((aryEdgesInfo%lEdge(1) .ne. -1.5) &
       .or.  (aryEdgesInfo%lEdge(2) .ne. -3.5) &
       .or.  (aryEdgesInfo%lEdge(3) .ne. -2.5)) then
      print *, 'Wrong lEdge'
      errExit = .true.
    else if ((aryEdgesInfo%uEdge(1) .ne.  3.5) &
       .or.  (aryEdgesInfo%uEdge(2) .ne.  0.5) &
       .or.  (aryEdgesInfo%uEdge(3) .ne.  1.5)) then
      print *, 'Wrong uEdge'
      errExit = .true.
    end if

    if (errExit) then
      call printAryEdgesInfo(aryEdgesInfo)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "GetAryEdgesInfo022a"
    print *,"=== "//errHead//": 3-dim Array (lboundIndex=(4,5,6), style=wcs)."

    aryEdgesInfo = getAryEdgesInfo(ary3d, lboundIndex=(/4,5,6/), style='wcs')
    errExit = .false.
    if (aryEdgesInfo%aryDimension .ne. 3) then
      print *, 'Wrong aryDimension'
      errExit = .true.
    else if (size(aryEdgesInfo%arySize) .ne. 3) then
      print *, 'Wrong size'
      errExit = .true.
    else if ((aryEdgesInfo%arySize(1) .ne. 3) &
       .or.  (aryEdgesInfo%arySize(2) .ne. 4) &
       .or.  (aryEdgesInfo%arySize(3) .ne. 5)) then
      print *, 'Wrong arySize'
      errExit = .true.
    else if ((aryEdgesInfo%lboundIndex(1) .ne. 4) &
       .or.  (aryEdgesInfo%lboundIndex(2) .ne. 5) &
       .or.  (aryEdgesInfo%lboundIndex(3) .ne. 6)) then
      print *, 'Wrong lboundIndex'
      errExit = .true.
    else if ((aryEdgesInfo%lEdge(1) .ne.  3.5) &
       .or.  (aryEdgesInfo%lEdge(2) .ne.  4.5) &
       .or.  (aryEdgesInfo%lEdge(3) .ne.  5.5)) then
      print *, 'Wrong lEdge'
      errExit = .true.
    else if ((aryEdgesInfo%uEdge(1) .ne.  6.5) &
       .or.  (aryEdgesInfo%uEdge(2) .ne.  8.5) &
       .or.  (aryEdgesInfo%uEdge(3) .ne. 10.5)) then
      print *, 'Wrong uEdge'
      errExit = .true.
    end if

    if (errExit) then
      call printAryEdgesInfo(aryEdgesInfo)
      call error("err"//errHead, '')
    end if

    deallocate(ary3d, lEdge, uEdge)	!-------- Important!


  end subroutine test_getAryEdgesInfo


  !---------------------------------------------------------------------
  subroutine test_getAryStatInfo
  !---------------------------------------------------------------------
  ! Tests of getAryStatInfo() in array_utils.

    implicit none

    type(aryStatInfoDoubleT) :: aryInfo
    type(aryStatInfoSingleT) :: aryInfoSingle
    type(aryStatInfoInt32T)  :: aryInfoInt32
    type(aryStatInfoInt16T)  :: aryInfoInt16
    type(aryStatInfoInt8T)   :: aryInfoInt8

    type(aryStatInfoFlagT) :: flagInfo

    real(double) :: arsimplest(3,3), arsimplest1(3)
    real(double) :: arsh2x3(2,3), arsh6(6)
    real(single) :: arsh2x3single(2,3),  arsh6single(6)
    integer(int32) :: arsh2x3int32(2,3), arsh6int32(6)
    integer(int16) :: arsh2x3int16(2,3), arsh6int16(6)
    integer(int8)  :: arsh2x3int8(2,3),  arsh6int8(6)
    logical :: arsh2x3mask(2,3), arsh6mask(6)
    real(single), allocatable :: arshBigSingle2d(:,:), arshBigSingle1d(:)
    logical :: isErr
    character(8)  :: errHead
  
    integer(int32), parameter :: sizeBigAry = 600
    real(single), parameter   :: constSgl = 2.9618206620E-01

    !-------------------------------------------------------------------
    ! Start:

    arsimplest(:,:)=3.0d0
    arsimplest1(:)=3.0d0
    arsh6int32 = (/ 1, 3, 5, 2, 4, 6 /)
    arsh6       = real(arsh6int32, double)
    arsh6single = real(arsh6int32, single)
    arsh6int16  = int(arsh6int32, int16)
    arsh6int8   = int(arsh6int32, int8)

    !----------------------------------------------------------
    errHead = "Stat001a"
    print *,"=== "//errHead//": Array of 3x3 of 3.0."
    aryInfo = getAryStatInfo(arsimplest)
    !call chkAryStatInfo(aryInfo, isErr, 3*3*3.0, 3.0, 0.0 &
    call chkAryStatInfoDouble(aryInfo, isErr, 3*3*3.0d0, 3.0d0, 0.0d0 &
       , 9, 9, 3.0d0, 3.0d0, status=0 &
      !, 8, 9, 3.0d0, 3.0d0, status=int(0, int16) &	! Causes error (for check).
       , isValLowerUsed=.false., isValUpperUsed=.false. &
       , isAreaLowerUsed=.false., isAreaUpperUsed=.false., isMaskUsed=.false.)
    if (isErr) then
      call printAryStatInfo(aryInfo)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Stat101a"
    print *,"=== "//errHead//": Array of 6 of 3.0."
    aryInfo = getAryStatInfo(arsimplest1)
    !call chkAryStatInfo(aryInfo, isErr, 3*3*3.0, 3.0, 0.0 &
    call chkAryStatInfoDouble(aryInfo, isErr, 3*3.0d0, 3.0d0, 0.0d0 &
       , 3, 3, 3.0d0, 3.0d0, status=0 &
      !, 8, 9, 3.0d0, 3.0d0, status=int(0, int16) &	! Causes error (for check).
       , isValLowerUsed=.false., isValUpperUsed=.false. &
       , isAreaLowerUsed=.false., isAreaUpperUsed=.false., isMaskUsed=.false.)
    if (isErr) then
      call printAryStatInfo(aryInfo)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Stat002a"
    print *,"=== "//errHead//": Array of (1,3,5  [x=1]"
    print *,"    :Simple-Test       2,4,6) [x=2]"
    print *,"                 ...[y=1,2,3]"  

    arsh2x3 = reshape( source=(/1,2,3,4,5,6/), shape=(/2,3/) )
    !print *," (1,2) = ", arsh2x3(1,2), " ; (2,1) = ", arsh2x3(2,1) !! => 3 ... 2
    aryInfo = getAryStatInfo(arsh2x3)

    call chkAryStatInfoDouble(aryInfo, isErr, 21d0, 21/6.0d0, 1.707825128d0 &
       , 6, 6, 1.0d0, 6.0d0, (/ 1,1 /), (/ 2,3 /), status=0 &
       , isValLowerUsed=.false., isValUpperUsed=.false. &
       , isAreaLowerUsed=.false., isAreaUpperUsed=.false., isMaskUsed=.false.)
    if (isErr) then
      call printAryStatInfo(aryInfo)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Stat102a"
    print *,"=== "//errHead//": Array (1, 3, 5, 2, 4, 6)."
    aryInfo = getAryStatInfo(arsh6)

    call chkAryStatInfoDouble(aryInfo, isErr, 21d0, 21/6.0d0, 1.707825128d0 &
       , 6, 6, 1.0d0, 6.0d0, (/ 1 /), (/ 6 /), status=0 &
       , isValLowerUsed=.false., isValUpperUsed=.false. &
       , isAreaLowerUsed=.false., isAreaUpperUsed=.false., isMaskUsed=.false.)
    if (isErr) then
      call printAryStatInfo(aryInfo)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Stat003a"
    print *,"=== "//errHead//": Array of (1,3,5  [x=1]"
    print *,"    :LowerThre=2       2,4,6) [x=2]"
    print *,"                 ...[y=1,2,3]"  

    aryInfo = getAryStatInfo(arsh2x3, valLower=2d0)

    call chkAryStatInfoDouble(aryInfo, isErr, 20d0, 20/5.0d0, real(sqrt(2.0),double) &
       , 6, 5, 2.0d0, 6.0d0, (/ 2,1 /), (/ 2,3 /), vallower=2d0, status=0 &
       , isValLowerUsed=.true., isValUpperUsed=.false. &
       , isAreaLowerUsed=.false., isAreaUpperUsed=.false., isMaskUsed=.false.)
    if (isErr) then
      call printAryStatInfo(aryInfo)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Stat103a"
    print *,"=== "//errHead//": Array (1, 3, 5, 2, 4, 6)."
    print *,"    :LowerThre=2"

    aryInfo = getAryStatInfo(arsh6, valLower=2d0)

    call chkAryStatInfoDouble(aryInfo, isErr, 20d0, 20/5.0d0, real(sqrt(2.0),double) &
       , 6, 5, 2.0d0, 6.0d0, (/ 4 /), (/ 6 /), vallower=2d0, status=0 &
       , isValLowerUsed=.true., isValUpperUsed=.false. &
       , isAreaLowerUsed=.false., isAreaUpperUsed=.false., isMaskUsed=.false.)
    if (isErr) then
      call printAryStatInfo(aryInfo)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Stat004a"
    print *,"=== "//errHead//": Array of (1,3,5  [x=1]"
    print *,"    :UpperThre=4       2,4,6) [x=2]"
    print *,"                 ...[y=1,2,3]"  

    aryInfo = getAryStatInfo(arsh2x3, valUpper=4d0)

    call chkAryStatInfoDouble(aryInfo, isErr, 10d0, 10d0/4, 1.11803399d0 &
       , 6, 4, 1.0d0, 4.0d0, (/ 1,1 /), (/ 2,2 /), valupper=4d0, status=0 &
       , isValLowerUsed=.false., isValUpperUsed=.true. &
       , isAreaLowerUsed=.false., isAreaUpperUsed=.false., isMaskUsed=.false.)
    if (isErr) then
      call printAryStatInfo(aryInfo)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Stat104a"
    print *,"=== "//errHead//": Array (1, 3, 5, 2, 4, 6)."
    print *,"    :UpperThre=4"

    aryInfo = getAryStatInfo(arsh6, valUpper=4d0)

    call chkAryStatInfoDouble(aryInfo, isErr, 10d0, 10d0/4, 1.11803399d0 &
       , 6, 4, 1.0d0, 4.0d0, (/ 1 /), (/ 5 /), valupper=4d0, status=0 &
       , isValLowerUsed=.false., isValUpperUsed=.true. &
       , isAreaLowerUsed=.false., isAreaUpperUsed=.false., isMaskUsed=.false.)
    if (isErr) then
      call printAryStatInfo(aryInfo)
      call error("err"//errHead, '')
    end if


    !----------------------------------------------------------
    errHead = "Stat005a"
    print *,"=== "//errHead//": Array of (1,3,5  [x=1]"
    print *,"    :LowerThre=2       2,4,6) [x=2]"
    print *,"    :UpperThre=4 ...[y=1,2,3]"  
    print *,"                ==>   (*,3,*  [x=1]"
    print *,"                       2,4,*) [x=2]"

    aryInfo = getAryStatInfo(arsh2x3, valLower=2d0, valUpper=4d0)

    call chkAryStatInfoDouble(aryInfo, isErr, 9d0, 9d0/3, 0.816496581d0 &
       , 6, 3, 2d0, 4d0, (/ 2,1 /), (/ 2,2 /), 2d0, 4d0, status=0 &
       , isValLowerUsed=.true., isValUpperUsed=.true. &
       , isAreaLowerUsed=.false., isAreaUpperUsed=.false., isMaskUsed=.false.)
    if (isErr) then
      call printAryStatInfo(aryInfo)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Stat105a"
    print *,"=== "//errHead//": Array (1, 3, 5, 2, 4, 6)."
    print *,"    :LowerThre=2"
    print *,"    :UpperThre=4"  

    aryInfo = getAryStatInfo(arsh6, valLower=2d0, valUpper=4d0)

    call chkAryStatInfoDouble(aryInfo, isErr, 9d0, 9d0/3, 0.816496581d0 &
       , 6, 3, 2d0, 4d0, (/ 4 /), (/ 5 /), 2d0, 4d0, status=0 &
       , isValLowerUsed=.true., isValUpperUsed=.true. &
       , isAreaLowerUsed=.false., isAreaUpperUsed=.false., isMaskUsed=.false.)
    if (isErr) then
      call printAryStatInfo(aryInfo)
      call error("err"//errHead, '')
    end if


    !----------------------------------------------------------
    errHead = "Stat006a"
    print *,"=== "//errHead//": Array of (1,3,5  [x=1]"
    print *,"    :LowerThre=2       2,4,6) [x=2]"
    print *,"    :MinAreaIndices=(2,1)"
    print *,"                ==>   (*,*,*  [x=1]"
    print *,"                       2,4,6) [x=2]"

    aryInfo = getAryStatInfo(arsh2x3, valLower=2d0, minAreaIndices=(/2,1/))

    call chkAryStatInfoDouble(aryInfo, isErr, 12d0, 12d0/3, 1.63299316d0 &
       , 6, 3, 2d0, 6d0, (/ 2,1 /), (/ 2,3 /), vallower=2d0 &
       , minareaindices=(/2,1/), status=0 &
       , isValLowerUsed=.true., isValUpperUsed=.false. &
       , isAreaLowerUsed=.true., isAreaUpperUsed=.false., isMaskUsed=.false.)
    if (isErr) then
      call printAryStatInfo(aryInfo)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Stat106a"
    print *,"=== "//errHead//": Array (1, 3, 5, 2, 4, 6)."
    print *,"    :LowerThre=2"
    print *,"    :MinAreaIndices=(2)"

    aryInfo = getAryStatInfo(arsh6, valLower=2d0, minAreaIndices=(/2/))

    call chkAryStatInfoDouble(aryInfo, isErr, 20d0, 20/5.0d0, real(sqrt(2.0),double) &
       , 6, 5, 2.0d0, 6.0d0, (/ 4 /), (/ 6 /), vallower=2d0, status=0 &
       , isValLowerUsed=.true., isValUpperUsed=.false. &
       , isAreaLowerUsed=.true., isAreaUpperUsed=.false., isMaskUsed=.false.)
    if (isErr) then
      call printAryStatInfo(aryInfo)
      call error("err"//errHead, '')
    end if


    !----------------------------------------------------------
    errHead = "Stat007a"
    print *,"=== "//errHead//": Array of (1,3,5  [x=1]"
    print *,"    :LowerThre=2       2,4,6) [x=2]"
    print *,"    :arMaskIn=arsh2x3mask"
    print *,"                ==>   (1,3,5  [x=1]"
    print *,"                       2,*,6) [x=2]"

    arsh2x3mask = .true.
    arsh2x3mask(2,2) = .false.
    aryInfo = getAryStatInfo(arsh2x3, arMaskIn=arsh2x3mask)

    call chkAryStatInfoDouble(aryInfo, isErr, 17d0, 17d0/5, 1.8547236991d0 &
       , 6, 5, 1d0, 6d0, (/ 1,1 /), (/ 2,3 /), status=0 &
       , isValLowerUsed=.false., isValUpperUsed=.false. &
       , isAreaLowerUsed=.false., isAreaUpperUsed=.false., isMaskUsed=.true.)
    if (isErr) then
      call printAryStatInfo(aryInfo)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Stat107a"
    print *,"=== "//errHead//": Array (1, 3, 5, 2, 4, 6)."
    print *,"    :LowerThre=2"
    print *,"    :arMaskIn=arsh6mask == (T,T,T,T,F,T)"

    arsh6mask = .true.
    arsh6mask(5) = .false.
    aryInfo = getAryStatInfo(arsh6, arMaskIn=arsh6mask)

    call chkAryStatInfoDouble(aryInfo, isErr, 17d0, 17d0/5, 1.8547236991d0 &
       , 6, 5, 1d0, 6d0, (/ 1 /), (/ 6 /), status=0 &
       , isValLowerUsed=.false., isValUpperUsed=.false. &
       , isAreaLowerUsed=.false., isAreaUpperUsed=.false., isMaskUsed=.true.)
    if (isErr) then
      call printAryStatInfo(aryInfo)
      call error("err"//errHead, '')
    end if


    !----------------------------------------------------------
    errHead = "Stat008a"
    print *,"=== "//errHead//": Array of (1,3,5  [x=1]"
    print *,"    :UpperThre=8       2,4,6) [x=2]"
    print *,"    :MaxAreaIndices=(2,2)"
    print *,"    :arMaskIn=arsh2x3mask"
    print *,"                ==>   (*,3,*  [x=1]"
    print *,"                       2,4,*) [x=2]"

    arsh2x3mask = .true.
    arsh2x3mask(1,1) = .false.
    aryInfo = getAryStatInfo(arsh2x3, arMaskIn=arsh2x3mask, valUpper=8d0, maxAreaIndices=(/2,2/))

    call chkAryStatInfoDouble(aryInfo, isErr, 9d0, 9d0/3, 0.816496581d0 &
       , 6, 3, 2d0, 4d0, (/ 2,1 /), (/ 2,2 /), valupper=8d0 &
       , maxareaindices=(/2,2/), status=0 &
       , isValLowerUsed=.false., isValUpperUsed=.true. &
       , isAreaLowerUsed=.false., isAreaUpperUsed=.true., isMaskUsed=.true.)
    if (isErr) then
      call printAryStatInfo(aryInfo)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Stat108a"
    print *,"=== "//errHead//": Array (1, 3, 5, 2, 4, 6)."
    print *,"    :UpperThre=8"
    print *,"    :MaxAreaIndices=(4)"
    print *,"    :arMaskIn=arsh6mask == (F,T,T,T,T,T)"

    arsh6mask = .true.
    arsh6mask(1) = .false.
    aryInfo = getAryStatInfo(arsh6, arMaskIn=arsh6mask, valUpper=8d0, maxAreaIndices=(/4/))

    call chkAryStatInfoDouble(aryInfo, isErr, 10d0, 10d0/3, 1.2472191289d0 &
       , 6, 3, 2d0, 5d0, (/ 4 /), (/ 3 /), valupper=8d0 &
       , maxareaindices=(/ 4 /), status=0 &
       , isValLowerUsed=.false., isValUpperUsed=.true. &
       , isAreaLowerUsed=.false., isAreaUpperUsed=.true., isMaskUsed=.true.)
    if (isErr) then
      call printAryStatInfo(aryInfo)
      call error("err"//errHead, '')
    end if


    !----------------------------------------------------------
    errHead = "Stat009a"
    print *,"=== "//errHead//": Array of (1,3,5  [x=1]"
    print *,"    :UpperThre=8       2,4,6) [x=2]"
    print *,"    :MaxAreaIndices=(2,2)"
    print *,"    :arMaskIn=arsh2x3mask"
    print *,"                ==>   (*,3,*  [x=1]"
    print *,"                       2,4,*) [x=2]"
    print *,"    :flagInfo=[F,T,F,T,T]"

    arsh2x3mask = .true.
    arsh2x3mask(1,1) = .false.
    flagInfo%isValLowerUsed=.false.	! though present()
    flagInfo%isValUpperUsed=.true.
    flagInfo%isAreaLowerUsed=.false.
    flagInfo%isAreaUpperUsed=.true.
    flagInfo%isMaskUsed=.true.

    aryInfo = getAryStatInfo(arsh2x3, arMaskIn=arsh2x3mask, vallower=8d1, valUpper=8d0 &
       , maxAreaIndices=(/2,2/), flagInfo=flagInfo)

    call chkAryStatInfoDouble(aryInfo, isErr, 9d0, 9d0/3, 0.816496581d0 &
       , 6, 3, 2d0, 4d0, (/ 2,1 /), (/ 2,2 /), valupper=8d0 &
       , maxareaindices=(/2,2/), status=0 &
       , isValLowerUsed=.false., isValUpperUsed=.true. &
       , isAreaLowerUsed=.false., isAreaUpperUsed=.true., isMaskUsed=.true.)
    if (isErr) then
      call printAryStatInfo(aryInfo)
      call error("err"//errHead, '')
    end if


    !----------------------------------------------------------
    errHead = "Stat018a"
    print *,"=== "//errHead//": Array of (1,3,5  [x=1]  [Single]"
    print *,"    :UpperThre=8       2,4,6) [x=2]"
    print *,"    :MaxAreaIndices=(2,2)"
    print *,"    :arMaskIn=arsh2x3mask"
    print *,"                ==>   (*,3,*  [x=1]"
    print *,"                       2,4,*) [x=2]"

    arsh2x3single = reshape( source=(/1,2,3,4,5,6/), shape=(/2,3/) )

    arsh2x3mask = .true.
    arsh2x3mask(1,1) = .false.
    aryInfoSingle = getAryStatInfo(arsh2x3single &
       , arMaskIn=arsh2x3mask, valUpper=8.0, maxAreaIndices=(/2,2/))

    call chkAryStatInfoSingle(aryInfoSingle, isErr, 9.0, 9.0/3, 0.816496581 &
       , 6, 3, 2.0, 4.0, (/ 2,1 /), (/ 2,2 /), valupper=8.0 &
       , maxareaindices=(/2,2/), status=0 &
       , isValLowerUsed=.false., isValUpperUsed=.true. &
       , isAreaLowerUsed=.false., isAreaUpperUsed=.true., isMaskUsed=.true.)
    if (isErr) then
      call printAryStatInfo(aryInfoSingle)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Stat118a"
    print *,"=== "//errHead//": Array (1, 3, 5, 2, 4, 6) [Single]."
    print *,"    :UpperThre=8"
    print *,"    :MaxAreaIndices=(4)"
    print *,"    :arMaskIn=arsh6mask == (F,T,T,T,T,T)"

    arsh6mask = .true.
    arsh6mask(1) = .false.
    aryInfoSingle = getAryStatInfo(arsh6single &
       , arMaskIn=arsh6mask, valUpper=8.0, maxAreaIndices=(/4/))

    call chkAryStatInfoSingle(aryInfoSingle, isErr, 10.0, 10.0/3, 1.2472191289 &
       , 6, 3, 2.0, 5.0, (/ 4 /), (/ 3 /), valupper=8.0 &
       , maxareaindices=(/ 4 /), status=0 &
       , isValLowerUsed=.false., isValUpperUsed=.true. &
       , isAreaLowerUsed=.false., isAreaUpperUsed=.true., isMaskUsed=.true.)
    if (isErr) then
      call printAryStatInfo(aryInfoSingle)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Stat311a"
    print *,"=== "//errHead//": Large array for mean and sigma [Single, 1d]:", constSgl 
    ! This used to be affected by a rounded error before Ver.4.26.

    allocate(arshBigSingle1d(sizeBigAry*sizeBigAry))
    arshBigSingle1d = constSgl 
    aryInfoSingle = getAryStatInfo(arshBigSingle1d)

    call chkAryStatInfoSingle(aryInfoSingle, isErr, mean=constSgl, sigma=0.0, precisionIn=5)
    if (isErr) then
      call printAryStatInfo(aryInfoSingle)
      call error("err"//errHead, '')
    end if

    deallocate(arshBigSingle1d)

    !----------------------------------------------------------
    errHead = "Stat318a"
    print *,"=== "//errHead//": Large array for mean and sigma [Single, 2d]:", constSgl
    ! This used to be affected by a rounded error before Ver.4.26.

    allocate(arshBigSingle2d(sizeBigAry,sizeBigAry))
    arshBigSingle2d = constSgl 
    aryInfoSingle = getAryStatInfo(arshBigSingle2d)

    call chkAryStatInfoSingle(aryInfoSingle, isErr, mean=constSgl, sigma=0.0, precisionIn=5)
    if (isErr) then
      call printAryStatInfo(aryInfoSingle)
      call error("err"//errHead, '')
    end if

    deallocate(arshBigSingle2d)


    !----------------------------------------------------------
    errHead = "Stat028a"
    print *,"=== "//errHead//": Array of (1,3,5  [x=1]  [Int32]"
    print *,"    :UpperThre=8       2,4,6) [x=2]"
    print *,"    :MaxAreaIndices=(2,2)"
    print *,"    :arMaskIn=arsh2x3mask"
    print *,"                ==>   (*,3,*  [x=1]"
    print *,"                       2,4,*) [x=2]"

    arsh2x3int32 = reshape( source=(/1,2,3,4,5,6/), shape=(/2,3/) )

    arsh2x3mask = .true.
    arsh2x3mask(1,1) = .false.
    aryInfoInt32 = getAryStatInfo(arsh2x3int32 &
       , arMaskIn=arsh2x3mask, valUpper=8d0, maxAreaIndices=(/2,2/))

    call chkAryStatInfoInt32(aryInfoInt32, isErr, 9, 9d0/3, 0.816496581d0 &
       , 6, 3, 2, 4, (/ 2,1 /), (/ 2,2 /), valupper=8d0 &
       , maxareaindices=(/2,2/), status=0 &
       , isValLowerUsed=.false., isValUpperUsed=.true. &
       , isAreaLowerUsed=.false., isAreaUpperUsed=.true., isMaskUsed=.true.)
    if (isErr) then
      call printAryStatInfo(aryInfoInt32)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Stat128a"
    print *,"=== "//errHead//": Array (1, 3, 5, 2, 4, 6) [Int32]."
    print *,"    :UpperThre=8"
    print *,"    :MaxAreaIndices=(4)"
    print *,"    :arMaskIn=arsh6mask == (F,T,T,T,T,T)"

    arsh6mask = .true.
    arsh6mask(1) = .false.
    aryInfoInt32 = getAryStatInfo(arsh6int32 &
       , arMaskIn=arsh6mask, valUpper=8d0, maxAreaIndices=(/4/))

    call chkAryStatInfoInt32(aryInfoInt32, isErr, 10, 10d0/3, 1.2472191289d0 &
       , 6, 3, 2, 5, (/ 4 /), (/ 3 /), valupper=8d0 &
       , maxareaindices=(/4/), status=0 &
       , isValLowerUsed=.false., isValUpperUsed=.true. &
       , isAreaLowerUsed=.false., isAreaUpperUsed=.true., isMaskUsed=.true.)
    if (isErr) then
      call printAryStatInfo(aryInfoInt32)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Stat038a"
    print *,"=== "//errHead//": Array of (1,3,5  [x=1]  [Int16]"
    print *,"    :UpperThre=8       2,4,6) [x=2]"
    print *,"    :MaxAreaIndices=(2,2)"
    print *,"    :arMaskIn=arsh2x3mask"
    print *,"                ==>   (*,3,*  [x=1]"
    print *,"                       2,4,*) [x=2]"

    arsh2x3int16 = reshape( source=(/1,2,3,4,5,6/), shape=(/2,3/) )

    arsh2x3mask = .true.
    arsh2x3mask(1,1) = .false.
    aryInfoInt16 = getAryStatInfo(arsh2x3int16 &
       , arMaskIn=arsh2x3mask, valUpper=8d0, maxAreaIndices=(/2,2/))

    call chkAryStatInfoInt16(aryInfoInt16, isErr, 9, 9d0/3, 0.816496581d0 &
       , 6, 3, int(2, int16), int(4, int16), (/ 2,1 /), (/ 2,2 /), valupper=8d0 &
       , maxareaindices=(/2,2/), status=0 &
       , isValLowerUsed=.false., isValUpperUsed=.true. &
       , isAreaLowerUsed=.false., isAreaUpperUsed=.true., isMaskUsed=.true.)
    if (isErr) then
      call printAryStatInfo(aryInfoInt16)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Stat138a"
    print *,"=== "//errHead//": Array (1, 3, 5, 2, 4, 6) [Int16]."
    print *,"    :UpperThre=8"
    print *,"    :MaxAreaIndices=(4)"
    print *,"    :arMaskIn=arsh6mask == (F,T,T,T,T,T)"

    arsh6mask = .true.
    arsh6mask(1) = .false.
    aryInfoInt16 = getAryStatInfo(arsh6int16 &
       , arMaskIn=arsh6mask, valUpper=8d0, maxAreaIndices=(/4/))

    call chkAryStatInfoInt16(aryInfoInt16, isErr, 10, 10d0/3, 1.2472191289d0 &
       , 6, 3, int(2, int16), int(5, int16), (/ 4 /), (/ 3 /), valupper=8d0 &
       , maxareaindices=(/4/), status=0 &
       , isValLowerUsed=.false., isValUpperUsed=.true. &
       , isAreaLowerUsed=.false., isAreaUpperUsed=.true., isMaskUsed=.true.)
    if (isErr) then
      call printAryStatInfo(aryInfoInt16)
      call error("err"//errHead, '')
    end if


    !----------------------------------------------------------
    errHead = "Stat039a"
    print *,"=== "//errHead//": Array of (1,3,5  [x=1]  [Int16]"
    print *,"    :UpperThre=8       2,4,6) [x=2]"
    print *,"    :MaxAreaIndices=(2,2)"
    print *,"    :arMaskIn=arsh2x3mask"
    print *,"                ==>   (*,3,*  [x=1]"
    print *,"                       2,4,*) [x=2]"
    print *,"    :flagInfo=[F,T,F,T,T]"

    arsh2x3int16 = reshape( source=(/1,2,3,4,5,6/), shape=(/2,3/) )

    arsh2x3mask = .true.
    arsh2x3mask(1,1) = .false.
    flagInfo%isValLowerUsed=.false.	! though present()
    flagInfo%isValUpperUsed=.true.
    flagInfo%isAreaLowerUsed=.false.
    flagInfo%isAreaUpperUsed=.true.
    flagInfo%isMaskUsed=.true.

    aryInfoInt16 = getAryStatInfo(arsh2x3int16 &
       , arMaskIn=arsh2x3mask, vallower=8d1, valUpper=8d0 &
       , maxAreaIndices=(/2,2/), flagInfo=flagInfo)

    call chkAryStatInfoInt16(aryInfoInt16, isErr, 9, 9d0/3, 0.816496581d0 &
       , 6, 3, int(2, int16), int(4, int16), (/ 2,1 /), (/ 2,2 /), valupper=8d0 &
       , maxareaindices=(/2,2/), status=0 &
       , isValLowerUsed=.false., isValUpperUsed=.true. &
       , isAreaLowerUsed=.false., isAreaUpperUsed=.true., isMaskUsed=.true.)
    if (isErr) then
      call printAryStatInfo(aryInfoInt16)
      call error("err"//errHead, '')
    end if


    !----------------------------------------------------------
    errHead = "Stat048a"
    print *,"=== "//errHead//": Array of (1,3,5  [x=1]  [Int8]"
    print *,"    :UpperThre=8       2,4,6) [x=2]"
    print *,"    :MaxAreaIndices=(2,2)"
    print *,"    :arMaskIn=arsh2x3mask"
    print *,"                ==>   (*,3,*  [x=1]"
    print *,"                       2,4,*) [x=2]"

    arsh2x3int8 = reshape( source=(/1,2,3,4,5,6/), shape=(/2,3/) )

    arsh2x3mask = .true.
    arsh2x3mask(1,1) = .false.
    aryInfoInt8 = getAryStatInfo(arsh2x3int8 &
       , arMaskIn=arsh2x3mask, valUpper=8d0, maxAreaIndices=(/2,2/))

    call chkAryStatInfoInt8(aryInfoInt8, isErr, 9, 9d0/3, 0.816496581d0 &
       , 6, 3, int(2, int8), int(4, int8), (/ 2,1 /), (/ 2,2 /), valupper=8d0 &
       , maxareaindices=(/2,2/), status=0 &
       , isValLowerUsed=.false., isValUpperUsed=.true. &
       , isAreaLowerUsed=.false., isAreaUpperUsed=.true., isMaskUsed=.true.)
    if (isErr) then
      call printAryStatInfo(aryInfoInt8)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Stat148a"
    print *,"=== "//errHead//": Array (1, 3, 5, 2, 4, 6) [Int8]."
    print *,"    :UpperThre=8"
    print *,"    :MaxAreaIndices=(4)"
    print *,"    :arMaskIn=arsh6mask == (F,T,T,T,T,T)"

    arsh6mask = .true.
    arsh6mask(1) = .false.
    aryInfoInt8 = getAryStatInfo(arsh6int8 &
       , arMaskIn=arsh6mask, valUpper=8d0, maxAreaIndices=(/4/))

    call chkAryStatInfoInt8(aryInfoInt8, isErr, 10, 10d0/3, 1.2472191289d0 &
       , 6, 3, int(2, int8), int(5, int8), (/ 4 /), (/ 3 /), valupper=8d0 &
       , maxareaindices=(/4/), status=0 &
       , isValLowerUsed=.false., isValUpperUsed=.true. &
       , isAreaLowerUsed=.false., isAreaUpperUsed=.true., isMaskUsed=.true.)
    if (isErr) then
      call printAryStatInfo(aryInfoInt8)
      call error("err"//errHead, '')
    end if


    !----------------------------------------------------------
    errHead = "Stat211a"
    print *,"=== "//errHead//": 1d-Array of 0 (double)"
    print *,"    :arMaskIn=arsh6mask (=.false.)"

    arsh6 = 0.0d0
    arsh6mask = .false.
    aryInfo = getAryStatInfo(arsh6 &
       , arMaskIn=arsh6mask)	! Used to cause abort (fatal error): e.g., "Subscript 1 of ARIN (value 0) is out of range (1:512)"

    if (aryInfo%flag%status .ne. -1) then	! Abnormal end.
      call printAryStatInfo(aryInfo)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Stat211b"
    print *,"=== "//errHead//": 1d-Array of 7 (double) with invalid min/maxAreaIndices"

    arsh6 = 7.0d0
    arsh6mask = .false.
    aryInfo = getAryStatInfo(arsh6 &
       , minAreaIndices=(/2/), maxAreaIndices=(/1/))

    if (aryInfo%flag%status > -4) then	! (= -5) Abnormal end.
      call printAryStatInfo(aryInfo)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Stat212a"
    print *,"=== "//errHead//": 2d-Array of 0 (double)"
    print *,"    :arMaskIn=arsh2x3mask (=.false.)"
    ! print *,"                ==>   (*,3,*  [x=1]"
    ! print *,"                       2,4,*) [x=2]"

    arsh2x3 = 0.0d0
    arsh2x3mask = .false.
    ! arsh2x3mask(1,1) = .false.
    aryInfo = getAryStatInfo(arsh2x3 &
       , arMaskIn=arsh2x3mask)	! Used to cause abort (fatal error): e.g., "Subscript 1 of ARIN (value 0) is out of range (1:512)"

    ! call chkAryStatInfoInt8(aryInfoDouble, isErr, 9, 9d0/3, 0.816496581d0 &
    !    , 6, 3, int(2, int8), int(4, int8), (/ 2,1 /), (/ 2,2 /), valupper=8d0 &
    !    , maxareaindices=(/2,2/), status=0 &
    !    , isValLowerUsed=.false., isValUpperUsed=.true. &
    !    , isAreaLowerUsed=.false., isAreaUpperUsed=.true., isMaskUsed=.true.)

    if (aryInfo%flag%status .ne. -1) then	! Abnormal end.
      call printAryStatInfo(aryInfo)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Stat212b"
    print *,"=== "//errHead//": 2d-Array of 7 (double) with invalid min/maxAreaIndices"

    arsh2x3 = 7.0d0
    aryInfo = getAryStatInfo(arsh2x3 &
       , minAreaIndices=(/2,2/), maxAreaIndices=(/2,1/))

    if (aryInfo%flag%status > -4) then	! (= -5) Abnormal end.
      call printAryStatInfo(aryInfo)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Stat212c"
    print *,"=== "//errHead//": 2d-Array of 7 (double) with 2 invalid min/maxAreaIndices"

    arsh2x3 = 7.0d0
    aryInfo = getAryStatInfo(arsh2x3 &
       , minAreaIndices=(/2,2/), maxAreaIndices=(/1,1/))

    if (aryInfo%flag%status > -8) then	! (= -9) Abnormal end.
      call printAryStatInfo(aryInfo)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Stat221a"
    print *,"=== "//errHead//": 1d-Array of 0 (single)"
    print *,"    :arMaskIn=arsh6mask (=.false.)"

    arsh6single = 0.0d0
    arsh6mask = .false.
    aryInfoSingle = getAryStatInfo(arsh6single &
       , arMaskIn=arsh6mask)	! Used to cause abort (fatal error): e.g., "Subscript 1 of ARIN (value 0) is out of range (1:512)"

    if (aryInfoSingle%flag%status .ne. -1) then	! Abnormal end.
      call printAryStatInfo(aryInfoSingle)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Stat222a"
    print *,"=== "//errHead//": 2d-Array of 0 (single)"
    print *,"    :arMaskIn=arsh2x3mask (=.false.)"

    arsh2x3single = 0.0
    arsh2x3mask = .false.
    aryInfoSingle = getAryStatInfo(arsh2x3single &
       , arMaskIn=arsh2x3mask)	! Used to cause abort (fatal error): e.g., "Subscript 1 of ARIN (value 0) is out of range (1:512)"

    if (aryInfoSingle%flag%status .ne. -1) then	! Abnormal end.
      call printAryStatInfo(aryInfoSingle)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Stat222b"
    print *,"=== "//errHead//": 2d-Array of 7 (single) with 2 invalid min/maxAreaIndices"

    arsh2x3single = 7.0
    aryInfoSingle = getAryStatInfo(arsh2x3single &
       , minAreaIndices=(/2,2/), maxAreaIndices=(/1,1/))

    if (aryInfo%flag%status > -8) then	! (= -9) Abnormal end.
      call printAryStatInfo(aryInfoSingle)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Stat231a"
    print *,"=== "//errHead//": 1d-Array of 0 (int32)"
    print *,"    :arMaskIn=arsh6mask (=.false.)"

    arsh6int32 = 0.0d0
    arsh6mask = .false.
    aryInfoInt32 = getAryStatInfo(arsh6int32 &
       , arMaskIn=arsh6mask)	! Used to cause abort (fatal error): e.g., "Subscript 1 of ARIN (value 0) is out of range (1:512)"

    if (aryInfoInt32%flag%status .ne. -1) then	! Abnormal end.
      call printAryStatInfo(aryInfoInt32)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Stat232a"
    print *,"=== "//errHead//": 2d-Array of 0 (int32)"
    print *,"    :arMaskIn=arsh2x3mask (=.false.)"

    arsh2x3int32 = 0.0
    arsh2x3mask = .false.
    aryInfoInt32 = getAryStatInfo(arsh2x3int32 &
       , arMaskIn=arsh2x3mask)	! Used to cause abort (fatal error): e.g., "Subscript 1 of ARIN (value 0) is out of range (1:512)"

    if (aryInfoInt32%flag%status .ne. -1) then	! Abnormal end.
      call printAryStatInfo(aryInfoInt32)
      call error("err"//errHead, '')
    end if


    !----------------------------------------------------------
    errHead = "Stat241a"
    print *,"=== "//errHead//": 1d-Array of 0 (int16)"
    print *,"    :arMaskIn=arsh6mask (=.false.)"

    arsh6int16 = 0.0d0
    arsh6mask = .false.
    aryInfoInt16 = getAryStatInfo(arsh6int16 &
       , arMaskIn=arsh6mask)	! Used to cause abort (fatal error): e.g., "Subscript 1 of ARIN (value 0) is out of range (1:512)"

    if (aryInfoInt16%flag%status .ne. -1) then	! Abnormal end.
      call printAryStatInfo(aryInfoInt16)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Stat242a"
    print *,"=== "//errHead//": 2d-Array of 0 (int16)"
    print *,"    :arMaskIn=arsh2x3mask (=.false.)"

    arsh2x3int16 = 0.0
    arsh2x3mask = .false.
    aryInfoInt16 = getAryStatInfo(arsh2x3int16 &
       , arMaskIn=arsh2x3mask)	! Used to cause abort (fatal error): e.g., "Subscript 1 of ARIN (value 0) is out of range (1:512)"

    if (aryInfoInt16%flag%status .ne. -1) then	! Abnormal end.
      call printAryStatInfo(aryInfoInt16)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Stat242b"
    print *,"=== "//errHead//": 2d-Array of 7 (int16)"

    arsh2x3int16 = 0.0
    arsh2x3mask = .false.
    aryInfoInt16 = getAryStatInfo(arsh2x3int16 &
       , minAreaIndices=(/2,2/), maxAreaIndices=(/1,1/))

    if (aryInfo%flag%status > -8) then	! (= -9) Abnormal end.
      call printAryStatInfo(aryInfoInt16)
      call error("err"//errHead, '')
    end if


    !----------------------------------------------------------
    errHead = "Stat251a"
    print *,"=== "//errHead//": 1d-Array of 0 (int8)"
    print *,"    :arMaskIn=arsh6mask (=.false.)"

    arsh6int8 = 0.0d0
    arsh6mask = .false.
    aryInfoInt8 = getAryStatInfo(arsh6int8 &
       , arMaskIn=arsh6mask)	! Used to cause abort (fatal error): e.g., "Subscript 1 of ARIN (value 0) is out of range (1:512)"

    if (aryInfoInt8%flag%status .ne. -1) then	! Abnormal end.
      call printAryStatInfo(aryInfoInt8)
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "Stat252a"
    print *,"=== "//errHead//": 2d-Array of 0 (int8)"
    print *,"    :arMaskIn=arsh2x3mask (=.false.)"

    arsh2x3int8 = 0.0
    arsh2x3mask = .false.
    aryInfoInt8 = getAryStatInfo(arsh2x3int8 &
       , arMaskIn=arsh2x3mask)	! Used to cause abort (fatal error): e.g., "Subscript 1 of ARIN (value 0) is out of range (1:512)"

    if (aryInfoInt8%flag%status .ne. -1) then	! Abnormal end.
      call printAryStatInfo(aryInfoInt8)
      call error("err"//errHead, '')
    end if


    ! subroutine chkAryStatInfoDouble(aryInfo, isErr, totalsum, mean, sigma &
    !   , totalentry, validentry &
    !   , minval, maxval, minindices, maxindices, vallower, valupper &
    !   , minareaindices, maxareaindices, status &
    !   , isValLowerUsed, isValUpperUsed, isAreaLowerUsed, isAreaUpperUsed &
    !   , isMaskUsed, precisionIn)

  end subroutine test_getAryStatInfo


  !---------------------------------------------------------------------
  subroutine chkAryStatInfoDouble(aryInfo, isErr, totalsum, mean, sigma &
    , totalentry, validentry &
    , minval, maxval, minindices, maxindices, vallower, valupper &
    , minareaindices, maxareaindices, status &
    , isValLowerUsed, isValUpperUsed, isAreaLowerUsed, isAreaUpperUsed &
    , isMaskUsed, precisionIn)
  !---------------------------------------------------------------------
  ! Check whether the aryInfo (<== type(aryStatInfoDoubleT)) is as expected.
  !
  ! NOTE: If a non-initialised value is specified either as one of arguments
  ! or in aryInfo, then this subroutine may cause the Fortran
  ! "*** Arithmetic exception:  - aborting".  A typical case is
  ! the argument "vallower" is given when aryInfo%vallower is undefined.

    implicit none
    !integer, parameter :: rankAry = 2

    type(aryStatInfoDoubleT), intent(in) :: aryInfo
    logical, intent(out) :: isErr
    real(double), intent(in), optional   :: totalsum, mean, sigma
    integer(int32), intent(in), optional :: totalentry, validentry
    real(double), intent(in), optional   :: minval, maxval
    !integer(int32), dimension(rankAry), intent(in), optional :: minindices, maxindices
    integer(int32), dimension(:), intent(in), optional :: minindices, maxindices
    real(double), intent(in), optional   :: vallower, valupper
    !integer(int32), dimension(rankAry), intent(in), optional :: minareaindices, maxareaindices
    integer(int32), dimension(:), intent(in), optional :: minareaindices, maxareaindices
    integer, intent(in), optional :: status	! Normal
    logical, intent(in), optional :: isValLowerUsed, isValUpperUsed &
       , isAreaLowerUsed, isAreaUpperUsed, isMaskUsed
    integer, intent(in), optional :: precisionIn

    integer :: i, precision

    !-------------------------------------------------------------------
    ! Start:

    isErr = .false.
    precision = 3

    if (present(precisionIn)) then
      precision = precisionIn
    end if

    if (present(totalsum)) then
      if (.not. isNearlyEqual(aryInfo%totalsum, totalsum, precision)) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", ES23.16, " / ", ES23.16)') &
           "totalsum", totalsum, aryInfo%totalsum
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(mean)) then
      if (.not. isNearlyEqual(aryInfo%mean, mean, precision)) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", ES23.16, " / ", ES23.16)') &
           "mean", mean, aryInfo%mean
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(sigma)) then
      if (.not. isNearlyEqual(aryInfo%sigma, sigma, precision)) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", ES23.16, " / ", ES23.16)') &
           "sigma", sigma, aryInfo%sigma
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(totalentry)) then
      if (aryInfo%totalentry .ne. totalentry) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", I21, " / ", I21)') &
           "totalentry", totalentry, aryInfo%totalentry
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(validentry)) then
      if (aryInfo%validentry .ne. validentry) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", I21, " / ", I21)') &
           "validentry", validentry, aryInfo%validentry
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(minval)) then
      if (.not. isNearlyEqual(aryInfo%minval, minval, precision)) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", ES23.16, " / ", ES23.16)') &
           "minval", minval, aryInfo%minval
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(maxval)) then
      if (.not. isNearlyEqual(aryInfo%maxval, maxval, precision)) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", ES23.16, " / ", ES23.16)') &
           "maxval", maxval, aryInfo%maxval
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(minindices)) then
      do i=1,size(minindices)	!rankAry
        if (aryInfo%minindices(i) .ne. minindices(i)) then
          isErr = .true.
          write(errStr, '(A, "(", I1, ") : Correct / Reality = ", I21, " / ", I21)') &
             "minindices", i, minindices(i), aryInfo%minindices(i)
          call message(AppMsg, Verbose)
        end if
      end do
    end if

    if (present(maxindices)) then
      do i=1,size(maxindices)	!rankAry
        if (aryInfo%maxindices(i) .ne. maxindices(i)) then
          isErr = .true.
          write(errStr, '(A, "(", I1, ") : Correct / Reality = ", I21, " / ", I21)') &
             "maxindices", i, maxindices(i), aryInfo%maxindices(i)
          call message(AppMsg, Verbose)
        end if
      end do
    end if

    if (present(vallower)) then
      if (.not. isNearlyEqual(aryInfo%vallower, vallower, precision)) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", ES23.16, " / ", ES23.16)') &
           "vallower", vallower, aryInfo%vallower
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(valupper)) then
      if (.not. isNearlyEqual(aryInfo%valupper, valupper, precision)) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", ES23.16, " / ", ES23.16)') &
           "valupper", valupper, aryInfo%valupper
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(minareaindices)) then
      do i=1,size(minareaindices)	!rankAry
        if (aryInfo%minareaindices(i) .ne. minareaindices(i)) then
          isErr = .true.
          write(errStr, '(A, "(", I1, ") : Correct / Reality = ", I21, " / ", I21)') &
             "minareaindices", i, minareaindices(i), aryInfo%minareaindices(i)
          call message(AppMsg, Verbose)
        end if
      end do
    end if

    if (present(maxareaindices)) then
      do i=1,size(maxareaindices)	!rankAry
        if (aryInfo%maxareaindices(i) .ne. maxareaindices(i)) then
          isErr = .true.
          write(errStr, '(A, "(", I1, ") : Correct / Reality = ", ES23.16, " / ", ES23.16)') &
             "maxareaindices", i, maxareaindices(i), aryInfo%maxareaindices(i)
          call message(AppMsg, Verbose)
        end if
      end do
    end if

    if (present(status)) then
      if (aryInfo%flag%status .ne. status) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", I8, " / ", I8)') &
           "status", status, aryInfo%flag%status
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(isValLowerUsed)) then
      if (aryInfo%flag%isValLowerUsed .neqv. isValLowerUsed) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", L2, " / ", L2)') &
           "isValLowerUsed", isValLowerUsed, aryInfo%flag%isValLowerUsed
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(isValUpperUsed)) then
      if (aryInfo%flag%isValUpperUsed .neqv. isValUpperUsed) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", L2, " / ", L2)') &
           "isValUpperUsed", isValUpperUsed, aryInfo%flag%isValUpperUsed
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(isAreaLowerUsed)) then
      if (aryInfo%flag%isAreaLowerUsed .neqv. isAreaLowerUsed) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", L2, " / ", L2)') &
           "isAreaLowerUsed", isAreaLowerUsed, aryInfo%flag%isAreaLowerUsed
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(isAreaUpperUsed)) then
      if (aryInfo%flag%isAreaUpperUsed .neqv. isAreaUpperUsed) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", L2, " / ", L2)') &
           "isAreaUpperUsed", isAreaUpperUsed, aryInfo%flag%isAreaUpperUsed
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(isMaskUsed)) then
      if (aryInfo%flag%isMaskUsed .neqv. isMaskUsed) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", L2, " / ", L2)') &
           "isMaskUsed", isMaskUsed, aryInfo%flag%isMaskUsed
        call message(AppMsg, Verbose)
      end if
    end if

    if (isErr) write(errStr, '()')

  end subroutine chkAryStatInfoDouble


  !---------------------------------------------------------------------
  subroutine chkAryStatInfoSingle(aryInfo, isErr, totalsum, mean, sigma &
    , totalentry, validentry &
    , minval, maxval, minindices, maxindices, vallower, valupper &
    , minareaindices, maxareaindices, status &
    , isValLowerUsed, isValUpperUsed, isAreaLowerUsed, isAreaUpperUsed &
    , isMaskUsed, precisionIn)
  !---------------------------------------------------------------------
  ! See the comments in  subroutine chkAryStatInfoDouble().

    implicit none
    !integer, parameter :: rankAry = 2

    type(aryStatInfoSingleT), intent(in) :: aryInfo
    logical, intent(out) :: isErr
    real(single), intent(in), optional   :: totalsum, mean, sigma
    integer(int32), intent(in), optional :: totalentry, validentry
    real(single), intent(in), optional   :: minval, maxval
    integer(int32), dimension(:), intent(in), optional :: minindices, maxindices
    real(single), intent(in), optional   :: vallower, valupper
    integer(int32), dimension(:), intent(in), optional :: minareaindices, maxareaindices
    integer, intent(in), optional :: status	! Normal
    logical, intent(in), optional :: isValLowerUsed, isValUpperUsed &
       , isAreaLowerUsed, isAreaUpperUsed, isMaskUsed
    integer, intent(in), optional :: precisionIn

    integer :: i, precision

    !-------------------------------------------------------------------
    ! Start:

    isErr = .false.
    precision = 3

    if (present(precisionIn)) then
      precision = precisionIn
    end if

    if (present(totalsum)) then
      if (.not. isNearlyEqual(aryInfo%totalsum, totalsum, precision)) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", ES23.16, " / ", ES23.16)') &
           "totalsum", totalsum, aryInfo%totalsum
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(mean)) then
      if (.not. isNearlyEqual(aryInfo%mean, mean, precision)) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", ES23.16, " / ", ES23.16)') &
           "mean", mean, aryInfo%mean
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(sigma)) then
      if (.not. isNearlyEqual(aryInfo%sigma, sigma, precision)) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", ES23.16, " / ", ES23.16)') &
           "sigma", sigma, aryInfo%sigma
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(totalentry)) then
      if (aryInfo%totalentry .ne. totalentry) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", I21, " / ", I21)') &
           "totalentry", totalentry, aryInfo%totalentry
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(validentry)) then
      if (aryInfo%validentry .ne. validentry) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", I21, " / ", I21)') &
           "validentry", validentry, aryInfo%validentry
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(minval)) then
      if (.not. isNearlyEqual(aryInfo%minval, minval, precision)) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", ES23.16, " / ", ES23.16)') &
           "minval", minval, aryInfo%minval
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(maxval)) then
      if (.not. isNearlyEqual(aryInfo%maxval, maxval, precision)) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", ES23.16, " / ", ES23.16)') &
           "maxval", maxval, aryInfo%maxval
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(minindices)) then
      do i=1,size(minindices)
        if (aryInfo%minindices(i) .ne. minindices(i)) then
          isErr = .true.
          write(errStr, '(A, "(", I1, ") : Correct / Reality = ", I21, " / ", I21)') &
             "minindices", i, minindices(i), aryInfo%minindices(i)
          call message(AppMsg, Verbose)
        end if
      end do
    end if

    if (present(maxindices)) then
      do i=1,size(maxindices)
        if (aryInfo%maxindices(i) .ne. maxindices(i)) then
          isErr = .true.
          write(errStr, '(A, "(", I1, ") : Correct / Reality = ", I21, " / ", I21)') &
             "maxindices", i, maxindices(i), aryInfo%maxindices(i)
          call message(AppMsg, Verbose)
        end if
      end do
    end if

    if (present(vallower)) then
      if (.not. isNearlyEqual(aryInfo%vallower, vallower, precision)) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", ES23.16, " / ", ES23.16)') &
           "vallower", vallower, aryInfo%vallower
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(valupper)) then
      if (.not. isNearlyEqual(aryInfo%valupper, valupper, precision)) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", ES23.16, " / ", ES23.16)') &
           "valupper", valupper, aryInfo%valupper
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(minareaindices)) then
      do i=1,size(minareaindices)
        if (aryInfo%minareaindices(i) .ne. minareaindices(i)) then
          isErr = .true.
          write(errStr, '(A, "(", I1, ") : Correct / Reality = ", I21, " / ", I21)') &
             "minareaindices", i, minareaindices(i), aryInfo%minareaindices(i)
          call message(AppMsg, Verbose)
        end if
      end do
    end if

    if (present(maxareaindices)) then
      do i=1,size(maxareaindices)
        if (aryInfo%maxareaindices(i) .ne. maxareaindices(i)) then
          isErr = .true.
          write(errStr, '(A, "(", I1, ") : Correct / Reality = ", ES23.16, " / ", ES23.16)') &
             "maxareaindices", i, maxareaindices(i), aryInfo%maxareaindices(i)
          call message(AppMsg, Verbose)
        end if
      end do
    end if

    if (present(status)) then
      if (aryInfo%flag%status .ne. status) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", I8, " / ", I8)') &
           "status", status, aryInfo%flag%status
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(isValLowerUsed)) then
      if (aryInfo%flag%isValLowerUsed .neqv. isValLowerUsed) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", L2, " / ", L2)') &
           "isValLowerUsed", isValLowerUsed, aryInfo%flag%isValLowerUsed
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(isValUpperUsed)) then
      if (aryInfo%flag%isValUpperUsed .neqv. isValUpperUsed) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", L2, " / ", L2)') &
           "isValUpperUsed", isValUpperUsed, aryInfo%flag%isValUpperUsed
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(isAreaLowerUsed)) then
      if (aryInfo%flag%isAreaLowerUsed .neqv. isAreaLowerUsed) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", L2, " / ", L2)') &
           "isAreaLowerUsed", isAreaLowerUsed, aryInfo%flag%isAreaLowerUsed
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(isAreaUpperUsed)) then
      if (aryInfo%flag%isAreaUpperUsed .neqv. isAreaUpperUsed) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", L2, " / ", L2)') &
           "isAreaUpperUsed", isAreaUpperUsed, aryInfo%flag%isAreaUpperUsed
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(isMaskUsed)) then
      if (aryInfo%flag%isMaskUsed .neqv. isMaskUsed) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", L2, " / ", L2)') &
           "isMaskUsed", isMaskUsed, aryInfo%flag%isMaskUsed
        call message(AppMsg, Verbose)
      end if
    end if

    if (isErr) write(errStr, '()')

  end subroutine chkAryStatInfoSingle


  !---------------------------------------------------------------------
  subroutine chkAryStatInfoInt32(aryInfo, isErr, totalsum, mean, sigma &
    , totalentry, validentry &
    , minval, maxval, minindices, maxindices, vallower, valupper &
    , minareaindices, maxareaindices, status &
    , isValLowerUsed, isValUpperUsed, isAreaLowerUsed, isAreaUpperUsed &
    , isMaskUsed, precisionIn)
  !---------------------------------------------------------------------
  ! See the comments in  subroutine chkAryStatInfoDouble().

    implicit none
    !integer, parameter :: rankAry = 2

    type(aryStatInfoInt32T), intent(in) :: aryInfo
    logical, intent(out) :: isErr
    integer(int32), intent(in), optional   :: totalsum
    real(double), intent(in), optional   :: mean, sigma
    integer(int32), intent(in), optional :: totalentry, validentry
    integer(int32), intent(in), optional   :: minval, maxval
    integer(int32), dimension(:), intent(in), optional :: minindices, maxindices
    real(double), intent(in), optional   :: vallower, valupper
    integer(int32), dimension(:), intent(in), optional :: minareaindices, maxareaindices
    integer, intent(in), optional :: status	! Normal
    logical, intent(in), optional :: isValLowerUsed, isValUpperUsed &
       , isAreaLowerUsed, isAreaUpperUsed, isMaskUsed
    integer, intent(in), optional :: precisionIn

    integer :: i, precision

    !-------------------------------------------------------------------
    ! Start:

    isErr = .false.
    precision = 3

    if (present(precisionIn)) then
      precision = precisionIn
    end if

    if (present(totalsum)) then
      if (.not. isNearlyEqual(aryInfo%totalsum, totalsum, precision)) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", I23, " / ", I23)') &
           "totalsum", totalsum, aryInfo%totalsum
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(mean)) then
      if (.not. isNearlyEqual(aryInfo%mean, mean, precision)) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", ES23.16, " / ", ES23.16)') &
           "mean", mean, aryInfo%mean
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(sigma)) then
      if (.not. isNearlyEqual(aryInfo%sigma, sigma, precision)) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", ES23.16, " / ", ES23.16)') &
           "sigma", sigma, aryInfo%sigma
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(totalentry)) then
      if (aryInfo%totalentry .ne. totalentry) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", I21, " / ", I21)') &
           "totalentry", totalentry, aryInfo%totalentry
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(validentry)) then
      if (aryInfo%validentry .ne. validentry) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", I21, " / ", I21)') &
           "validentry", validentry, aryInfo%validentry
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(minval)) then
      if (.not. isNearlyEqual(aryInfo%minval, minval, precision)) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", I23, " / ", I23)') &
           "minval", minval, aryInfo%minval
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(maxval)) then
      if (.not. isNearlyEqual(aryInfo%maxval, maxval, precision)) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", I23, " / ", I23)') &
           "maxval", maxval, aryInfo%maxval
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(minindices)) then
      do i=1,size(minindices)
        if (aryInfo%minindices(i) .ne. minindices(i)) then
          isErr = .true.
          write(errStr, '(A, "(", I1, ") : Correct / Reality = ", I21, " / ", I21)') &
             "minindices", i, minindices(i), aryInfo%minindices(i)
          call message(AppMsg, Verbose)
        end if
      end do
    end if

    if (present(maxindices)) then
      do i=1,size(maxindices)
        if (aryInfo%maxindices(i) .ne. maxindices(i)) then
          isErr = .true.
          write(errStr, '(A, "(", I1, ") : Correct / Reality = ", I21, " / ", I21)') &
             "maxindices", i, maxindices(i), aryInfo%maxindices(i)
          call message(AppMsg, Verbose)
        end if
      end do
    end if

    if (present(vallower)) then
      if (.not. isNearlyEqual(aryInfo%vallower, vallower, precision)) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", ES23.16, " / ", ES23.16)') &
           "vallower", vallower, aryInfo%vallower
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(valupper)) then
      if (.not. isNearlyEqual(aryInfo%valupper, valupper, precision)) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", ES23.16, " / ", ES23.16)') &
           "valupper", valupper, aryInfo%valupper
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(minareaindices)) then
      do i=1,size(minareaindices)
        if (aryInfo%minareaindices(i) .ne. minareaindices(i)) then
          isErr = .true.
          write(errStr, '(A, "(", I1, ") : Correct / Reality = ", I21, " / ", I21)') &
             "minareaindices", i, minareaindices(i), aryInfo%minareaindices(i)
          call message(AppMsg, Verbose)
        end if
      end do
    end if

    if (present(maxareaindices)) then
      do i=1,size(maxareaindices)
        if (aryInfo%maxareaindices(i) .ne. maxareaindices(i)) then
          isErr = .true.
          write(errStr, '(A, "(", I1, ") : Correct / Reality = ", ES23.16, " / ", ES23.16)') &
             "maxareaindices", i, maxareaindices(i), aryInfo%maxareaindices(i)
          call message(AppMsg, Verbose)
        end if
      end do
    end if

    if (present(status)) then
      if (aryInfo%flag%status .ne. status) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", I8, " / ", I8)') &
           "status", status, aryInfo%flag%status
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(isValLowerUsed)) then
      if (aryInfo%flag%isValLowerUsed .neqv. isValLowerUsed) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", L2, " / ", L2)') &
           "isValLowerUsed", isValLowerUsed, aryInfo%flag%isValLowerUsed
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(isValUpperUsed)) then
      if (aryInfo%flag%isValUpperUsed .neqv. isValUpperUsed) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", L2, " / ", L2)') &
           "isValUpperUsed", isValUpperUsed, aryInfo%flag%isValUpperUsed
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(isAreaLowerUsed)) then
      if (aryInfo%flag%isAreaLowerUsed .neqv. isAreaLowerUsed) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", L2, " / ", L2)') &
           "isAreaLowerUsed", isAreaLowerUsed, aryInfo%flag%isAreaLowerUsed
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(isAreaUpperUsed)) then
      if (aryInfo%flag%isAreaUpperUsed .neqv. isAreaUpperUsed) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", L2, " / ", L2)') &
           "isAreaUpperUsed", isAreaUpperUsed, aryInfo%flag%isAreaUpperUsed
        call message(AppMsg, Verbose)
      end if
    end if

    if (present(isMaskUsed)) then
      if (aryInfo%flag%isMaskUsed .neqv. isMaskUsed) then
        isErr = .true.
        write(errStr, '(A, ": Correct / Reality = ", L2, " / ", L2)') &
           "isMaskUsed", isMaskUsed, aryInfo%flag%isMaskUsed
        call message(AppMsg, Verbose)
      end if
    end if

    if (isErr) write(errStr, '()')

  end subroutine chkAryStatInfoInt32


  !---------------------------------------------------------------------
  subroutine chkAryStatInfoInt16(aryInfo, isErr, totalsum, mean, sigma &
    , totalentry, validentry &
    , minval, maxval, minindices, maxindices, vallower, valupper &
    , minareaindices, maxareaindices, status &
    , isValLowerUsed, isValUpperUsed, isAreaLowerUsed, isAreaUpperUsed &
    , isMaskUsed, precisionIn)
  !---------------------------------------------------------------------
  ! See the comments in  subroutine chkAryStatInfoDouble().

    implicit none
    !integer, parameter :: rankAry = 2

    type(aryStatInfoInt16T), intent(in) :: aryInfo
    logical, intent(out) :: isErr
    integer(int32), intent(in), optional   :: totalsum
    real(double), intent(in), optional   :: mean, sigma
    integer(int32), intent(in), optional :: totalentry, validentry
    integer(int16), intent(in), optional   :: minval, maxval
    integer(int32), dimension(:), intent(in), optional :: minindices, maxindices
    real(double), intent(in), optional   :: vallower, valupper
    integer(int32), dimension(:), intent(in), optional :: minareaindices, maxareaindices
    integer, intent(in), optional :: status	! Normal
    logical, intent(in), optional :: isValLowerUsed, isValUpperUsed &
       , isAreaLowerUsed, isAreaUpperUsed, isMaskUsed
    integer, intent(in), optional :: precisionIn

    type(aryStatInfoInt32T) :: aryInfoInt32

    aryInfoInt32 = convertAryStatIntToInt32(aryInfo)

    call chkAryStatInfoInt32(aryInfoInt32, isErr, totalsum, mean, sigma &
      , totalentry, validentry &
      , int(minval, int32), int(maxval, int32), minindices, maxindices, vallower, valupper &
      , minareaindices, maxareaindices, status &
      , isValLowerUsed, isValUpperUsed, isAreaLowerUsed, isAreaUpperUsed &
      , isMaskUsed, precisionIn)

  end subroutine chkAryStatInfoInt16


  !---------------------------------------------------------------------
  subroutine chkAryStatInfoInt8(aryInfo, isErr, totalsum, mean, sigma &
    , totalentry, validentry &
    , minval, maxval, minindices, maxindices, vallower, valupper &
    , minareaindices, maxareaindices, status &
    , isValLowerUsed, isValUpperUsed, isAreaLowerUsed, isAreaUpperUsed &
    , isMaskUsed, precisionIn)
  !---------------------------------------------------------------------
  ! See the comments in  subroutine chkAryStatInfoDouble().

    implicit none
    !integer, parameter :: rankAry = 2

    type(aryStatInfoInt8T), intent(in) :: aryInfo
    logical, intent(out) :: isErr
    integer(int32), intent(in), optional   :: totalsum
    real(double), intent(in), optional   :: mean, sigma
    integer(int32), intent(in), optional :: totalentry, validentry
    integer(int8), intent(in), optional   :: minval, maxval
    integer(int32), dimension(:), intent(in), optional :: minindices, maxindices
    real(double), intent(in), optional   :: vallower, valupper
    integer(int32), dimension(:), intent(in), optional :: minareaindices, maxareaindices
    integer, intent(in), optional :: status	! Normal
    logical, intent(in), optional :: isValLowerUsed, isValUpperUsed &
       , isAreaLowerUsed, isAreaUpperUsed, isMaskUsed
    integer, intent(in), optional :: precisionIn

    type(aryStatInfoInt32T) :: aryInfoInt32

    aryInfoInt32 = convertAryStatIntToInt32(aryInfo)

    call chkAryStatInfoInt32(aryInfoInt32, isErr, totalsum, mean, sigma &
      , totalentry, validentry &
      , int(minval, int32), int(maxval, int32), minindices, maxindices, vallower, valupper &
      , minareaindices, maxareaindices, status &
      , isValLowerUsed, isValUpperUsed, isAreaLowerUsed, isAreaUpperUsed &
      , isMaskUsed, precisionIn)

  end subroutine chkAryStatInfoInt8


  !---------------------------------------------------------------------
  subroutine test_getAnnularMaskAry
  !---------------------------------------------------------------------
  ! Tests of getAnnularMaskAry() in array_utils.

    implicit none

    real(double) :: centX, centY, rOuter, rInner
    logical, allocatable :: arMask(:,:)
    integer :: i

    ! logical :: isErr
    character(11)  :: errHead
    integer(int32), parameter :: sizeXDef=10, sizeYDef=10

    !-------------------------------------------------------------------
    ! Start:

    !----------------------------------------------------------
    errHead = "Annular001a"

    print *,"=== "//errHead//": Array of 10x10 with radius of 3 from (5,5)."
    centX = 5
    centY = 5
    rOuter = 3
    ! rInner
    arMask = getAnnularMaskAry(sizeXDef, sizeYDef, centX, centY, rOuter)

    if (size(arMask,1) .ne. sizeXDef) then
      print *, arMask
      call error("err"//errHead, 'sizeXDef')
    else if (size(arMask,2) .ne. sizeYDef) then
      print *, arMask
      call error("err"//errHead, 'sizeYDef')
    else if (.not. all(arMask(int(centX-rOuter+1):int(centX+rOuter-1), &
                              int(centY-rOuter+1):int(centY+rOuter-1)))) then
      print *, arMask
      call error("err"//errHead, 'Centre: false')
    else if (any(arMask(:, &
                        int(centY+rOuter+1):))) then
      print *, arMask
      call error("err"//errHead, 'Outer: true')
    end if

    ! Debug...
    do i=1,sizeYDef
      print *,arMask(:,i)
    end do

    deallocate(arMask)

    !----------------------------------------------------------
    errHead = "Annular002a"

    print *,"=== "//errHead//": Array of 10x10 with radii of 0-1 from (5,5)."
    centX = 5
    centY = 5
    rOuter = 1
    rInner = 0
    arMask = getAnnularMaskAry(sizeXDef, sizeYDef, centX, centY, rOuter, rInner)

    if (size(arMask,1) .ne. sizeXDef) then
      print *, arMask
      call error("err"//errHead, 'sizeXDef')
    else if (size(arMask,2) .ne. sizeYDef) then
      print *, arMask
      call error("err"//errHead, 'sizeYDef')
    else if (.not. arMask(5,5)) then
      print *, arMask
      call error("err"//errHead, 'arMask(5,5) == 1')
    else if (count(arMask) .ne. 1) then
      print *, arMask
      call error("err"//errHead, 'count == 1')
    end if

    !----------------------------------------------------------
    errHead = "Annular002b"

    print *,"=== "//errHead//": Array of 10x10 with radii of 2-4 from (5,5)."
    centX = 5
    centY = 5
    rOuter = 4
    rInner = 2
    arMask = getAnnularMaskAry(sizeXDef, sizeYDef, centX, centY, rOuter, rInner)

    if (size(arMask,1) .ne. sizeXDef) then
      print *, arMask
      call error("err"//errHead, 'sizeXDef')
    else if (size(arMask,2) .ne. sizeYDef) then
      print *, arMask
      call error("err"//errHead, 'sizeYDef')
    else if (.not. all(arMask(int(centX-rOuter+1):int(centX-rInner-1), &
                              int(centY)))) then
      print *, arMask
      call error("err"//errHead, 'Annular(Smaller-X): false')
    else if (.not. all(arMask(int(centX+rInner+1):int(centX+rOuter-1), &
                              int(centY)))) then
      print *, arMask
      call error("err"//errHead, 'Annular(larger-X): false')
    else if (.not. all(arMask(int(centX), &
                              int(centY-rOuter+1):int(centY-rInner-1)))) then
      print *, arMask
      call error("err"//errHead, 'Annular(Smaller-Y): false')
    else if (any(arMask(int(centX-rInner+1):int(centX+rInner-1), &
                        int(centY)))) then
      print *, arMask
      call error("err"//errHead, 'Inner(X): true')
    else if (any(arMask(int(centX), &
                        int(centY-rInner+1):int(centY+rInner-1)))) then
      print *, arMask
      call error("err"//errHead, 'Inner(Y): true')
    else if (any(arMask(:, &
                        int(centY+rOuter+1):))) then
      print *, arMask
      call error("err"//errHead, 'Outer: true')
    end if

    ! Debug...
    do i=1,sizeYDef
      print *,arMask(:,i)
    end do

    deallocate(arMask)

    !----------------------------------------------------------
    errHead = "Annular003a"

    print *,"=== "//errHead//": Array of 10x10 with radii of 2-4 from (1,5)."
    centX = 1
    centY = 5
    rOuter = 4
    rInner = 2
    arMask = getAnnularMaskAry(sizeXDef, sizeYDef, centX, centY, rOuter, rInner)

    if (size(arMask,1) .ne. sizeXDef) then
      print *, arMask
      call error("err"//errHead, 'sizeXDef')
    else if (size(arMask,2) .ne. sizeYDef) then
      print *, arMask
      call error("err"//errHead, 'sizeYDef')
    ! else if (.not. all(arMask(int(centX-rOuter+1):int(centX-rInner-1), &
    !                           int(centY)))) then
    !   print *, arMask
    !   call error("err"//errHead, 'Annular(Smaller-X): false')
    else if (.not. all(arMask(int(centX+rInner+1):int(centX+rOuter-1), &
                              int(centY)))) then
      print *, arMask
      call error("err"//errHead, 'Annular(larger-X): false')
    else if (.not. all(arMask(int(centX), &
                              int(centY-rOuter+1):int(centY-rInner-1)))) then
      print *, arMask
      call error("err"//errHead, 'Annular(Smaller-Y): false')
    else if (any(arMask(max(1,int(centX-rInner+1)):int(centX+rInner-1), &
                        int(centY)))) then
      print *, arMask
      call error("err"//errHead, 'Inner(X): true')
    else if (any(arMask(int(centX), &
                        max(1,int(centY-rInner+1)):int(centY+rInner-1)))) then
      print *, arMask
      call error("err"//errHead, 'Inner(Y): true')
    else if (any(arMask(:, &
                        int(centY+rOuter+1):))) then
      print *, arMask
      call error("err"//errHead, 'Outer(X): true')
    else if (any(arMask(int(centX+rOuter+1):, &
                        :))) then
      print *, arMask
      call error("err"//errHead, 'Outer(Y): true')
    end if

    ! Debug...
    do i=1,sizeYDef
      print *,arMask(:,i)
    end do

    deallocate(arMask)

    !----------------------------------------------------------
    errHead = "Annular004a"

    print *,"=== "//errHead//": Array of 10x10 with radii of 2-4 from (5.5,9.5)."
    centX = 5.5
    centY = 9.5
    rOuter = 4
    rInner = 2
    arMask = getAnnularMaskAry(sizeXDef, sizeYDef, centX, centY, rOuter, rInner)

    if (size(arMask,1) .ne. sizeXDef) then
      print *, arMask
      call error("err"//errHead, 'sizeXDef')
    else if (size(arMask,2) .ne. sizeYDef) then
      print *, arMask
      call error("err"//errHead, 'sizeYDef')
    else if (.not. all(arMask(int(centX-rOuter+1):int(centX-rInner-1), &
                              int(centY)))) then
      print *, arMask
      call error("err"//errHead, 'Annular(Smaller-X): false')
    else if (.not. all(arMask(int(centX+rInner+1):int(centX+rOuter-1), &
                              int(centY)))) then
      print *, arMask
      call error("err"//errHead, 'Annular(larger-X): false')
    else if (.not. all(arMask(int(centX), &
                              int(centY-rOuter+1):int(centY-rInner-1)))) then
      print *, arMask
      call error("err"//errHead, 'Annular(Smaller-Y): false')
    else if (any(arMask(max(1,int(centX-rInner+1)):int(centX+rInner-1), &
                        int(centY)))) then
      print *, arMask
      call error("err"//errHead, 'Inner(X): true')
    else if (any(arMask(int(centX), &
                        max(1,int(centY-rInner+1)):min(int(centY+rInner-1),sizeYDef)))) then
      print *, arMask
      call error("err"//errHead, 'Inner(Y): true')
    ! else if (any(arMask(:, &
    !                     int(centY+rOuter+1):))) then
    !   print *, arMask
    !   call error("err"//errHead, 'Outer(X): true')
    else if (any(arMask(int(centX+rOuter+1):, &
                        :))) then
      print *, arMask
      call error("err"//errHead, 'Outer(Y): true')
    end if

    ! Debug...
    do i=1,sizeYDef
      print *,arMask(:,i)
    end do

    deallocate(arMask)

  end subroutine test_getAnnularMaskAry


end program testarrayutils
