program testdalaux
  use dal
  use errorhandling
  use types
  use dal_aux

  implicit none

  integer, parameter :: ich = 256
  character(ich), parameter :: dalTestSetName = 'outfiles/daltest001a.ds'
  character(ich), parameter :: tabTestName  = "someTable"
  character(ich), parameter :: colTestName1      = "int32Column"
  character(ich), parameter :: colTestName2      = "real64Column"
  character(ich), parameter :: colTestNameReal32 = "real32Column"
  character(ich), parameter :: colTestNameInt16  = "int16Column"
  character(ich), parameter :: colTestNameInt8   = "int8Column"
  character(ich), parameter :: aryTestName1 = "array1"
  character(ich), parameter :: aryTestName2 = "array2"

  call test_getTypeName		! makeFitsFor_getTypeName() is called from this.
  call test_splitSetTabName	! This uses files created in test_getTypeName() (ie., by makeFitsFor_getTypeName()).

contains

  !---------------------------------------------------------------------
  subroutine test_getTypeName
  !---------------------------------------------------------------------
  ! Tests of getTypeName() in dal_aux

    implicit none

    type(DataSetT) :: set
    type(TableT)   :: tab
    integer :: iType
    character(13)  :: errHead

    !-------------------------------------------------------------------
    ! Start

    call makeFitsFor_getTypeName()

    set = dataSet(trim(dalTestSetName), READ)

    !----------------------------------------------------------
    errHead = "DalDtype0010a"
    print *,"=== "//errHead//": Image from Set (Default)."

    iType = getDataType(set)
    if (iType .ne. INTEGER32) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "DalDtype0020a"
    print *,"=== "//errHead//": Image from Set+0."

    iType = getDataType(set, 0)
    if (iType .ne. INTEGER32) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "DalDtype0030a"
    print *,"=== "//errHead//": Image from Set+1."

    iType = getDataType(set, 1)
    if (iType .ne. REAL64) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "DalDtype0110a"
    print *,"=== "//errHead//": Column(1) from Tab."

    tab = table(set, 2)	! 2nd extention

    iType = getDataType(tab, trim(colTestName1))
    if (iType .ne. INTEGER32) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "DalDtype0120a"
    print *,"=== "//errHead//": Column(2) from Tab."

    iType = getDataType(tab, trim(colTestName2))
    if (iType .ne. REAL64) then
      call error("err"//errHead, '')
    end if

    call release(tab)

    !----------------------------------------------------------
    errHead = "DalDtype0210a"
    print *,"=== "//errHead//": Column(1) from Set, ExtNum."

    iType = getDataType(set, extNum=2, colName=trim(colTestName1))
    if (iType .ne. INTEGER32) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "DalDtype0220a"
    print *,"=== "//errHead//": Column(2) from Set, ExtName."

    iType = getDataType(set, extName=trim(tabTestName), colName=trim(colTestName2))
    if (iType .ne. REAL64) then
      call error("err"//errHead, '')
    end if


    !----------------------------------------------------------
    errHead = "DalDtype0310a"
    print *,"=== "//errHead//": Image from Name (Default)."

    iType = getDataType(trim(dalTestSetName))
    if (iType .ne. INTEGER32) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "DalDtype0320a"
    print *,"=== "//errHead//": Image from (Name, 0)."

    iType = getDataType(trim(dalTestSetName), 0)
    if (iType .ne. INTEGER32) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "DalDtype0330a"
    print *,"=== "//errHead//": Image from (Name, 1)."

    iType = getDataType(trim(dalTestSetName), 1)
    if (iType .ne. REAL64) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "DalDtype0340a"
    print *,"=== "//errHead//": Column(1) from FitsName, ExtNum"

    iType = getDataType(trim(dalTestSetName), 2, colName=trim(colTestName1))
    if (iType .ne. INTEGER32) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "DalDtype0350a"
    print *,"=== "//errHead//": Column(2) from FitsName, extName"

    iType = getDataType(trim(dalTestSetName), extName=trim(tabTestName) &
       , colName=trim(colTestName2))
    if (iType .ne. REAL64) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "DalDtype0360a"
    print *,"=== "//errHead//": Column(2) from FitsName:extName"

    iType = getDataType(trim(dalTestSetName)//':'//trim(tabTestName) &
       , colName=trim(colTestName2))
    if (iType .ne. REAL64) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "DalDtype0370a"
    print *,"=== "//errHead//": Column(2) for Priority test; from FitsName:dummyName, extNum, extName"

    iType = getDataType(trim(dalTestSetName)//':DUMMY_NAME' &
       , extNum=999, extName=trim(tabTestName), colName=trim(colTestName2))
    if (iType .ne. REAL64) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "DalDtype0371a"
    print *,"=== "//errHead//": Column(2) for Priority test; from FitsName, extNum, extName"

    iType = getDataType(trim(dalTestSetName) &
       , extNum=999, extName=trim(tabTestName), colName=trim(colTestName2))
    if (iType .ne. REAL64) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "DalDtype0371a"
    print *,"=== "//errHead//": Column(2) for Priority test; from FitsName:extName, extNum"

    iType = getDataType(trim(dalTestSetName)//':'//trim(tabTestName) &
       , extNum=999, colName=trim(colTestName2))
    if (iType .ne. REAL64) then
      call error("err"//errHead, '')
    end if

!!!!!!!!! Test for the cases that SHOULD cause an error. !!!!!!!!!!!!!!!!
!
!     !----------------------------------------------------------
!     print *,"=== DalDtype9010a: Image from (Name, 8)."	! => Error(tooLargeExtNumInGetDataType)
! 
!     iType = getDataType(trim(dalTestSetName), 8)
! 
!     !----------------------------------------------------------
!     print *,"=== DalDtype9020a: Image from (Name, 2)."	! => Error(inconsistentExtForAryInGetDataType)
! 
!     iType = getDataType(trim(dalTestSetName), 2)
! 
!     !----------------------------------------------------------
!     print *,"=== DalDtype9030a: Column(0) from (Name, 0, Col)"	! => Error(inconsistentExtForAryInGetDataType)
! 
!     iType = getDataType(trim(dalTestSetName), 0, colName=trim(colTestName1))

    call release(set)

  end subroutine test_getTypeName


  !---------------------------------------------------------------------
  subroutine makeFitsFor_getTypeName
  !---------------------------------------------------------------------
  ! cf., addarray.f90
  ! Note: The file created here is used in testfitspultil_col as well.
  !   Therefore, if you want to change the following, be extra careful!

    implicit none

    !character(256), parameter :: setName = 'outfiles/daltest001a.ds'
    integer, dimension(3), parameter :: imgDims = (/ 3,4,2 /)

    type(DataSetT) :: set
    type(ArrayT)   :: arr1, arr2
    type(TableT)   :: tab
    type(ColumnT)  :: col
    integer(int32),  dimension(:,:,:), pointer :: a1
    real(double),    dimension(:,:,:), pointer :: a2
    integer(kind=int32), dimension(:), pointer :: i32
    integer(kind=int16), dimension(:), pointer :: i16
    integer(kind=int8),  dimension(:), pointer :: i8
    real(kind=double), dimension(:), pointer :: r64
    real(kind=single), dimension(:), pointer :: r32
    integer :: i,j,k,n

    !-------------------------------------------------------------------
    ! Start

    ! create a set
    set = dataSet(trim(dalTestSetName), CREATE)

    ! create images
    arr1 = addArray(set, aryTestName1, INTEGER32, dimensions=imgDims )
    arr2 = addArray(set, aryTestName2, REAL64,    dimensions=imgDims )

    ! fill with unique numbers
    a1 => int32Array3Data(arr1)
    a2 => real64Array3Data(arr2)

    n = 0
    do k=0,1
      do j=0,3
        do i=0,2 
          a1(i,j,k) = n
          a2(i,j,k) = a1(i,j,k) + 1.0d0
          n = n + 1
        end do
      end do
    end do

    call release(arr2)
    call release(arr1)

    ! create a table
    tab = addTable(set,trim(tabTestName),100)

    col = addColumn(tab,trim(colTestNameInt8),INTEGER8,units="s",comment="int8 column")
    i8 => int8Data(col)
    do i=0,numberOfRows(tab)-1
      i8(i) = 1*(i+1)
    end do
    call release(col)

    col = addColumn(tab,trim(colTestNameInt16),INTEGER16,units="m",comment="int16 column")
    i16 => int16Data(col)
    do i=0,numberOfRows(tab)-1
      i16(i) = 2*(i+1)
    end do
    call release(col)

    col = addColumn(tab,trim(colTestName1),INTEGER32,units="h",comment="int32 column")
    i32 => int32Data(col)
    do i=0,numberOfRows(tab)-1
      i32(i) = 3*(i+1)
    end do
    call release(col)

    col = addColumn(tab,trim(colTestNameReal32),REAL32,units="hms",comment="real32 column")
    r32 => real32Data(col)
    do i=0,numberOfRows(tab)-1
      r32(i) = 0.5*(i+1)
    end do
    call release(col)

    col = addColumn(tab,trim(colTestName2),REAL64,units="dms",comment="real64 column")
    r64 => real64Data(col)
    do i=0,numberOfRows(tab)-1
      r64(i) = 0.25d0*(i+1)
    end do
    call release(col)

    call release(tab)

    call release(set)

  end subroutine makeFitsFor_getTypeName


  !---------------------------------------------------------------------
  subroutine test_splitSetTabName
  !---------------------------------------------------------------------
  ! Tests of splitSetTabName() in dal_aux

    implicit none

    character(ich) :: inSetTabName, inSetName, inTabName
    character(ich) :: setName, tabName
    character(13)  :: errHead
    logical:: noColon, fUseBlock

    !-------------------------------------------------------------------
    ! Start

    !----------------------------------------------------------
    errHead = "DalSplit0010a"
    print *,"=== "//errHead//": Standard (filename:tabname)."

    inSetTabName = trim(dalTestSetName)//":"//trim(tabTestName)	! uses global vars
    call splitSetTabName(inSetTabName, setName, tabName)
    if (trim(dalTestSetName) .ne. trim(setName)  .or.&
        trim(tabTestName)    .ne. trim(tabName)) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "DalSplit0020a"
    print *,"=== "//errHead//": Standard (filename:tabname) with noColonFound."

    inSetTabName = trim(dalTestSetName)//":"//trim(tabTestName)	! uses global vars
    call splitSetTabName(inSetTabName, setName, tabName, noColonFound=noColon)
    if (trim(dalTestSetName) .ne. trim(setName)  .or.&
        trim(tabTestName)    .ne. trim(tabName)  .or.&
        noColon) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "DalSplit0030a"
    print *,"=== "//errHead//": Non-existent tabname (filename:DUMMY)."

    inTabName = 'DUMMY'
    inSetTabName = trim(dalTestSetName)//":"//trim(inTabName)	! uses global vars
    call splitSetTabName(inSetTabName, setName, tabName)
    if (trim(dalTestSetName) .ne. trim(setName)  .or.&
        trim(inTabName)      .ne. trim(tabName)) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "DalSplit0040a"
    print *,"=== "//errHead//": Non-existent tabname (filename:DUMMY) with noColonFound."

    inTabName = 'DUMMY'
    inSetTabName = trim(dalTestSetName)//":"//trim(inTabName)	! uses global vars
    call splitSetTabName(inSetTabName, setName, tabName, noColonFound=noColon)
    if (trim(dalTestSetName) .ne. trim(setName)  .or.&
        trim(inTabName)      .ne. trim(tabName)  .or.&
        noColon) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "DalSplit0050a"
    print *,"=== "//errHead//": Non-existent filename:tabname (DUMMY:DUMMY)."

    inSetName = 'DUMMY1'
    inTabName = 'DUMMY2'
    inSetTabName = trim(inSetName)//":"//trim(inTabName)
    call splitSetTabName(inSetTabName, setName, tabName)
    if (trim(inSetName) .ne. trim(setName)  .or.&
        trim(inTabName) .ne. trim(tabName)) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "DalSplit0060a"
    print *,"=== "//errHead//": Non-existent filename (DUMMY) without colon with noColonFound with useBlock=.false."

    fUseBlock = .false.
    inSetTabName = 'DUMMY'
    call splitSetTabName(inSetTabName, setName, tabName, noColonFound=noColon, useBlock=fUseBlock)
    if (trim(inSetTabName) .ne. trim(setName)  .or.&
        '' .ne. trim(tabName)  .or.&
        .not. noColon) then
      call error("err"//errHead, '')
    end if

    !----------------------------------------------------------
    errHead = "DalSplit0070a"
    print *,"=== "//errHead//": Non-existent filename (DUMMY) without colon with noColonFound with useBlock=Default(=.true.)"

    inSetTabName = 'DUMMY'
    call splitSetTabName(inSetTabName, setName, tabName, noColonFound=noColon)	! => Give a warning if Noisy.
    if (trim(inSetTabName) .ne. trim(setName)  .or.&
        '' .ne. trim(tabName)  .or.&
        .not. noColon) then
print *, trim(dalTestSetName)," ",trim(inTabName)
print *, trim(setName)," (",trim(tabName),") ",noColon
      call error("err"//errHead, '')
    end if

    ! !----------------------------------------------------------
    !!!!! => Should cause : error (missingFile)
    ! 
    ! errHead = "DalSplit0071a"
    ! print *,"=== "//errHead//": Non-existent filename (DUMMY) without colon without noColonFound with useBlock=.true."
    ! 
    ! fUseBlock = .true.
    ! inSetTabName = 'DUMMY'
    ! call splitSetTabName(inSetTabName, setName, tabName, useBlock=fUseBlock)	! => Cause an error
    ! if (trim(inSetTabName) .ne. trim(setName)  .or.&
    !     '' .ne. trim(tabName)  .or.&
    !     .not. noColon) then
    !   call error("err"//errHead, '')
    ! end if

    !----------------------------------------------------------
    errHead = "DalSplit0080a"
    print *,"=== "//errHead//": Existent filename (Filename) without colon with noColonFound with useBlock=.true."

    fUseBlock = .true.
    inSetTabName = trim(dalTestSetName)
    call splitSetTabName(inSetTabName, setName, tabName, noColonFound=noColon, useBlock=fUseBlock)
    if (trim(inSetTabName) .ne. trim(setName)  .or.&
        trim(aryTestName1) .ne. trim(tabName)  .or.&
        .not. noColon) then
      call error("err"//errHead, '')
    end if

  end subroutine test_splitSetTabName

end program testdalaux
